@@ -11,7 +11,7 @@ doc = ezdxf.new('R2010')
 msp = doc.modelspace()
 
 # using an UCS simplifies 3D operations, but UCS definition can happen later
-# calculating corner points in local (UCS) coordinates without Vector class
+# calculating corner points in local (UCS) coordinates without Vec3 class
 angle = math.radians(360 / 5)
 corners_ucs = [(math.cos(angle * n), math.sin(angle * n), 0) for n in range(5)]
 
@@ -23,8 +23,8 @@ corners_ucs = list(transformation_ucs.points_to_wcs(corners_ucs))
 location_ucs = UCS(origin=(0, 2, 2)).rotate_local_x(math.radians(-45))
 msp.add_polyline3d(
     points=corners_ucs,
+    close=True,
     dxfattribs={
-        'closed': True,
         'color': 1,
     }
 ).transform(location_ucs.matrix)
