@@ -4,25 +4,26 @@ import time
 from pathlib import Path
 import ezdxf
 from ezdxf.addons import iterdxf
-BIGFILE = Path(ezdxf.EZDXF_TEST_FILES) / 'GKB-R2010.dxf'
+
+BIGFILE = Path(ezdxf.EZDXF_TEST_FILES) / "GKB-R2010.dxf"
 # BIGFILE = Path(ezdxf.EZDXF_TEST_FILES) / 'ACAD_R2000.dxf'
-OUTDIR = Path('~/Desktop/Outbox').expanduser()
+OUTDIR = Path("~/Desktop/Outbox").expanduser()
 
 t0 = time.perf_counter()
 doc = iterdxf.opendxf(BIGFILE)
-line_exporter = doc.export(OUTDIR / 'lines.dxf')
-text_exporter = doc.export(OUTDIR / 'text.dxf')
-polyline_exporter = doc.export(OUTDIR / 'polyline.dxf')
-lwpolyline_exporter = doc.export(OUTDIR / 'lwpolyline.dxf')
+line_exporter = doc.export(OUTDIR / "lines.dxf")
+text_exporter = doc.export(OUTDIR / "text.dxf")
+polyline_exporter = doc.export(OUTDIR / "polyline.dxf")
+lwpolyline_exporter = doc.export(OUTDIR / "lwpolyline.dxf")
 try:
     for entity in doc.modelspace():
-        if entity.dxftype() == 'LINE':
+        if entity.dxftype() == "LINE":
             line_exporter.write(entity)
-        elif entity.dxftype() == 'TEXT':
+        elif entity.dxftype() == "TEXT":
             text_exporter.write(entity)
-        elif entity.dxftype() == 'POLYLINE':
+        elif entity.dxftype() == "POLYLINE":
             polyline_exporter.write(entity)
-        elif entity.dxftype() == 'LWPOLYLINE':
+        elif entity.dxftype() == "LWPOLYLINE":
             lwpolyline_exporter.write(entity)
 finally:
     line_exporter.close()
@@ -31,4 +32,4 @@ finally:
     lwpolyline_exporter.close()
     doc.close()
 
-print(f'Processing time: {time.perf_counter()-t0:.2f}s')
+print(f"Processing time: {time.perf_counter()-t0:.2f}s")
