@@ -1,10 +1,10 @@
-# Copyright (c) 2020, Manfred Moitzi
+# Copyright (c) 2020-2021, Manfred Moitzi
 # License: MIT License
 from pathlib import Path
 import random
 import ezdxf
 
-DIR = Path('~/Desktop/Outbox').expanduser()
+DIR = Path("~/Desktop/Outbox").expanduser()
 MAX_SIZE = 100
 
 doc = ezdxf.new()
@@ -21,4 +21,4 @@ for _ in range(30):
 msp.add_wipeout([(30, 30), (70, 70)])
 doc.set_modelspace_vport(center=(50, 50), height=MAX_SIZE)
 
-doc.saveas(DIR / 'random_wipeout.dxf')
+doc.saveas(DIR / "random_wipeout.dxf")
