@@ -1,12 +1,14 @@
 # Purpose: draw a maurer rose with polylines
 # https://en.wikipedia.org/wiki/Maurer_rose
-# Created: 13.10.2019
-# Copyright (c) 2019, Manfred Moitzi
+# Copyright (c) 2019-2021, Manfred Moitzi
 # License: MIT License
 from typing import Iterable, Tuple
+import pathlib
 import math
 import ezdxf
+from ezdxf import zoom
 
+DIR = pathlib.Path("~/Desktop/Outbox").expanduser()
 N = 6  # The rose has n petals if N is odd, and 2N petals if N is even.
 D = 71  # delta angle in degrees
 TWO_PI = math.pi * 2
@@ -14,7 +16,7 @@ STEP360 = TWO_PI / 360
 
 
 def maurer_rose(n: int, d: int, radius: float) -> Iterable[Tuple[float, float]]:
-    i = 0
+    i: float = 0.0
     while i < TWO_PI:
         k = i * d
         r = radius * math.sin(n * k)
@@ -26,11 +28,20 @@ def maurer_rose(n: int, d: int, radius: float) -> Iterable[Tuple[float, float]]:
 
 def main(filename: str, n: int, d: int) -> None:
     doc = ezdxf.new()
+    doc.layers.add("PETALS", color=1)
+    doc.layers.add("NET", color=5)
+
     msp = doc.modelspace()
-    msp.add_lwpolyline(maurer_rose(n, 1, 250), dxfattribs={'closed': True, 'color': 1})  # petal outline
-    msp.add_lwpolyline(maurer_rose(n, d, 250), dxfattribs={'closed': True, 'color': 5})  # net
+    msp.add_lwpolyline(
+        maurer_rose(n, 1, 250), close=True, dxfattribs={"layer": "PETALS"}
+    )
+    msp.add_lwpolyline(
+        maurer_rose(n, d, 250), close=True, dxfattribs={"layer": "NET"}
+    )
+
+    zoom.extents(msp)
     doc.saveas(filename)
 
 
-if __name__ == '__main__':
-    main('maurer_rose.dxf', N, D)
+if __name__ == "__main__":
+    main(str(DIR / "maurer_rose.dxf"), N, D)
