@@ -1,15 +1,14 @@
 # Purpose: open example files with big polyface models
-# Created: 23.04.2014
-# Copyright (c) 2014-2020, Manfred Moitzi
+# Copyright (c) 2014-2021, Manfred Moitzi
 # License: MIT License
 import time
-from pathlib import Path
+import pathlib
 
 import ezdxf
 from ezdxf.render import MeshVertexMerger
 
-SRCDIR = Path(r'D:\Source\dxftest\CADKitSamples')
-OUTDIR = Path('~/Desktop/Outbox').expanduser()
+SRCDIR = pathlib.Path(ezdxf.EZDXF_TEST_FILES) / "CADKitSamples"
+OUTDIR = pathlib.Path("~/Desktop/Outbox").expanduser()
 
 
 def optimize_polyfaces(polyfaces):
@@ -31,37 +30,45 @@ def optimize_polyfaces(polyfaces):
 
 def optimize(name: str):
     filename = SRCDIR / name
-    new_filename = OUTDIR / ('optimized_' + name)
-    print(f'opening DXF file: {filename}')
+    new_filename = OUTDIR / ("optimized_" + name)
+    print(f"opening DXF file: {filename}")
     start_time = time.time()
     doc = ezdxf.readfile(filename)
     msp = doc.modelspace()
     end_time = time.time()
-    print(f'time for reading: {end_time - start_time:.1f} seconds')
+    print(f"time for reading: {end_time - start_time:.1f} seconds")
     print(f"DXF version: {doc.dxfversion}")
     print(f"Database contains {len(doc.entitydb)} entities.")
-    polyfaces = (polyline for polyline in msp.query('POLYLINE') if polyline.is_poly_face_mesh)
+    polyfaces = (
+        polyline
+        for polyline in msp.query("POLYLINE")
+        if polyline.is_poly_face_mesh
+    )
     optimize_polyfaces(polyfaces)
 
-    print(f'saving DXF file: {new_filename}')
+    print(f"saving DXF file: {new_filename}")
     start_time = time.time()
     doc.saveas(new_filename)
     end_time = time.time()
-    print(f'time for saving: {end_time - start_time:.1f} seconds')
+    print(f"time for saving: {end_time - start_time:.1f} seconds")
 
 
 def save_as(name):
     filename = SRCDIR / name
 
-    print(f'opening DXF file: {filename}')
+    print(f"opening DXF file: {filename}")
     start_time = time.time()
     doc = ezdxf.readfile(filename)
     msp = doc.modelspace()
     end_time = time.time()
-    print(f'time for reading: {end_time - start_time:.1f} seconds')
+    print(f"time for reading: {end_time - start_time:.1f} seconds")
     print(f"DXF version: {doc.dxfversion}")
     print(f"Database contains {len(doc.entitydb)} entities.")
-    polyfaces = (polyline for polyline in msp.query('POLYLINE') if polyline.is_poly_face_mesh)
+    polyfaces = (
+        polyline
+        for polyline in msp.query("POLYLINE")
+        if polyline.is_poly_face_mesh
+    )
 
     # create a new documents
     doc1 = ezdxf.new()
@@ -70,31 +77,37 @@ def save_as(name):
     msp2 = doc2.modelspace()
     for polyface in polyfaces:
         b = MeshVertexMerger.from_polyface(polyface)
-        b.render(msp1, dxfattribs={
-            'layer': polyface.dxf.layer,
-            'color': polyface.dxf.color,
-        })
-        b.render_polyface(msp2, dxfattribs={
-            'layer': polyface.dxf.layer,
-            'color': polyface.dxf.color,
-        })
+        b.render_mesh(
+            msp1,
+            dxfattribs={
+                "layer": polyface.dxf.layer,
+                "color": polyface.dxf.color,
+            },
+        )
+        b.render_polyface(
+            msp2,
+            dxfattribs={
+                "layer": polyface.dxf.layer,
+                "color": polyface.dxf.color,
+            },
+        )
 
-    new_filename = OUTDIR / ('mesh_' + name)
-    print(f'saving as mesh DXF file: {new_filename}')
+    new_filename = OUTDIR / ("mesh_" + name)
+    print(f"saving as mesh DXF file: {new_filename}")
     start_time = time.time()
     doc1.saveas(new_filename)
     end_time = time.time()
-    print(f'time for saving: {end_time - start_time:.1f} seconds')
+    print(f"time for saving: {end_time - start_time:.1f} seconds")
 
-    new_filename = OUTDIR / ('recreated_polyface_' + name)
-    print(f'saving as polyface DXF file: {new_filename}')
+    new_filename = OUTDIR / ("recreated_polyface_" + name)
+    print(f"saving as polyface DXF file: {new_filename}")
     start_time = time.time()
     doc2.saveas(new_filename)
     end_time = time.time()
-    print(f'time for saving: {end_time - start_time:.1f} seconds')
+    print(f"time for saving: {end_time - start_time:.1f} seconds")
 
 
-if __name__ == '__main__':
-    optimize('fanuc-430-arm.dxf')
-    optimize('cnc machine.dxf')
-    save_as('fanuc-430-arm.dxf')
\ No newline at end of file
+if __name__ == "__main__":
+    optimize("fanuc-430-arm.dxf")
+    optimize("cnc machine.dxf")
+    save_as("fanuc-430-arm.dxf")
