@@ -1,11 +1,12 @@
-# Copyright (c) 2020, Manfred Moitzi
+# Copyright (c) 2020-2021, Manfred Moitzi
 # License: MIT License
 
 from pathlib import Path
 import math
 import ezdxf
+from ezdxf import zoom
 
-DIR = Path('~/Desktop/Outbox').expanduser()
+DIR = Path("~/Desktop/Outbox").expanduser()
 
 doc = ezdxf.new()
 msp = doc.modelspace()
@@ -16,12 +17,12 @@ ellipse = msp.add_ellipse(
     ratio=0.5,
     start_param=0,
     end_param=math.tau,
-    dxfattribs={'layer': 'ellipse'},
+    dxfattribs={"layer": "ellipse"},
 )
 
-spline = ellipse.to_spline(replace=False, layout=msp)
-spline.dxf.layer = 'B-spline'
+spline = ellipse.to_spline(replace=False)
+spline.dxf.layer = "B-spline"
 spline.dxf.color = 1
-doc.set_modelspace_vport(2)
 
-doc.saveas(DIR / 'spline_from_ellipse.dxf')
+zoom.extents(msp)
+doc.saveas(DIR / "spline_from_ellipse.dxf")
