@@ -1,12 +1,11 @@
 # Purpose: using DXF groups
-# Created: 15.07.2015
-# Copyright (c) 2015 Manfred Moitzi
+# Copyright (c) 2015-2021 Manfred Moitzi
 # License: MIT License
 import ezdxf
 
 
 def create_group():
-    doc = ezdxf.new('R2000')
+    doc = ezdxf.new("R2000")
     msp = doc.modelspace()
     # create a new unnamed group, in reality they have a name like '*Annnn' and group names have to be unique
     group = doc.groups.new()
@@ -16,11 +15,11 @@ def create_group():
         # or in a block
         g.append(msp.add_line((0, 0), (3, 0)))
         g.append(msp.add_circle((0, 0), radius=2))
-    doc.saveas('group.dxf')
+    doc.saveas("group.dxf")
 
 
 def read_group():
-    doc = ezdxf.readfile('group.dxf')
+    doc = ezdxf.readfile("group.dxf")
     for name, group in doc.groups:
         print(f"GROUP: {name}\n")
         for entity in group:
