@@ -1,10 +1,9 @@
 # Purpose: using true color and transparency
-# Created: 05.09.2018
-# Copyright (c) 2018 Manfred Moitzi
+# Copyright (c) 2018-2021, Manfred Moitzi
 # License: MIT License
 import ezdxf
 
-LAYER_NAME = 'Lines'
+LAYER_NAME = "Lines"
 
 # line weight im mm times 100, e.g. 0.13mm = 13
 # minimum line weight 0
@@ -19,8 +18,8 @@ def lines_with_lineweight(msp, x1, x2):
             (x1, y),
             (x2, y),
             dxfattribs={
-                'layer': LAYER_NAME,
-                'lineweight': lineweight,
+                "layer": LAYER_NAME,
+                "lineweight": lineweight,
             },
         )
 
@@ -31,11 +30,11 @@ def lines_with_default_weight(msp, x1, x2):
         msp.add_line(
             (x1, y),
             (x2, y),
-            dxfattribs={'layer': LAYER_NAME},
+            dxfattribs={"layer": LAYER_NAME},
         )
 
 
-doc = ezdxf.new('R2004')
+doc = ezdxf.new("R2004")
 msp = doc.modelspace()
 lines_layer = doc.layers.new(LAYER_NAME)
 # set default line width as enum
@@ -44,4 +43,4 @@ lines_layer.dxf.lineweight = 35
 lines_with_lineweight(msp, x1=0, x2=100)
 lines_with_default_weight(msp, x1=150, x2=250)
 
-doc.saveas('using_lineweight.dxf')
+doc.saveas("using_lineweight.dxf")
