@@ -1,5 +1,4 @@
-# Created: 13.02.2018
-# Copyright (C) 2011-2018, Manfred Moitzi
+# Copyright (C) 2011-2021, Manfred Moitzi
 # License: MIT License
 from .table import Table, CustomCell
 from .mtext import MText
@@ -8,3 +7,4 @@ from .sierpinski_pyramid import SierpinskyPyramid
 from .dimlines import LinearDimension, AngularDimension, ArcDimension, RadialDimension, dimstyles
 from .importer import Importer
 from .r12writer import r12writer
+from .mtxpl import MTextExplode
