@@ -1,11 +1,13 @@
+# Copyright (c) 2020-2021, Matthew Broadway
+# License: MIT License
 from typing import List
 
-from ezdxf.addons.drawing.backend import Backend
+from ezdxf.addons.drawing.backend import BackendInterface
 from ezdxf.addons.drawing.type_hints import Color
-from ezdxf.math import Vector
+from ezdxf.math import Vec3
 
 
-def draw_rect(points: List[Vector], color: Color, out: Backend):
+def draw_rect(points: List[Vec3], color: Color, out: BackendInterface):
     from ezdxf.addons.drawing import Properties
 
     props = Properties()
