@@ -1,24 +1,70 @@
-# Created: 17.05.2019
-# Copyright (c) 2019, Manfred Moitzi
+# Copyright (c) 2019-2021, Manfred Moitzi
 # License: MIT License
 from typing import TYPE_CHECKING, Iterable, List, Mapping, Set
 import json
 
 from ezdxf.sections.tables import TABLENAMES
 from ezdxf.lldxf.tags import Tags
+from ezdxf.entities import BoundaryPathType, EdgeType
 
 if TYPE_CHECKING:
     from ezdxf.eztypes import (
-        Insert, MText, LWPolyline, Polyline, Spline, Leader, Dimension, Image,
-        Mesh, Hatch, Wipeout,
+        Insert,
+        MText,
+        LWPolyline,
+        Polyline,
+        Spline,
+        Leader,
+        Dimension,
+        Image,
+        Mesh,
+        Hatch,
+        MPolygon,
+        Wipeout,
     )
     from ezdxf.eztypes import DXFEntity, Linetype, DXFTag, BlockLayout
+    from ezdxf.entities.polygon import DXFPolygon
 
-__all__ = ["entities_to_code", "block_to_code", "table_entries_to_code"]
+__all__ = [
+    "entities_to_code",
+    "block_to_code",
+    "table_entries_to_code",
+    "black",
+]
 
 
-def entities_to_code(entities: Iterable["DXFEntity"], layout: str = "layout",
-                     ignore: Iterable[str] = None) -> "Code":
+def black(code: str, line_length=88, fast: bool = True) -> str:
+    """Returns the source `code` as a single string formatted by `Black`_
+
+    Requires the installed `Black`_ formatter::
+
+        pip3 install black
+
+    Args:
+        code: source code
+        line_length: max. source code line length
+        fast: ``True`` for fast mode, ``False`` to check that the reformatted
+            code is valid
+
+    Raises:
+        ImportError: Black is not available
+
+    .. _black: https://pypi.org/project/black/
+
+    """
+
+    import black
+
+    mode = black.FileMode()
+    mode.line_length = line_length
+    return black.format_file_contents(code, fast=fast, mode=mode)
+
+
+def entities_to_code(
+    entities: Iterable["DXFEntity"],
+    layout: str = "layout",
+    ignore: Iterable[str] = None,
+) -> "Code":
     """
     Translates DXF entities into Python source code to recreate this entities
     by ezdxf.
@@ -38,8 +84,9 @@ def entities_to_code(entities: Iterable["DXFEntity"], layout: str = "layout",
     return code.code
 
 
-def block_to_code(block: "BlockLayout", drawing: str = "doc",
-                  ignore: Iterable[str] = None) -> "Code":
+def block_to_code(
+    block: "BlockLayout", drawing: str = "doc", ignore: Iterable[str] = None
+) -> "Code":
     """
     Translates a BLOCK into Python source code to recreate the BLOCK by ezdxf.
 
@@ -53,6 +100,7 @@ def block_to_code(block: "BlockLayout", drawing: str = "doc",
         :class:`Code`
 
     """
+    assert block.block is not None
     dxfattribs = _purge_handles(block.block.dxfattribs())
     block_name = dxfattribs.pop("name")
     base_point = dxfattribs.pop("base_point")
@@ -66,19 +114,20 @@ def block_to_code(block: "BlockLayout", drawing: str = "doc",
     return code.code
 
 
-def table_entries_to_code(entities: Iterable["DXFEntity"],
-                          drawing="doc") -> "Code":
+def table_entries_to_code(
+    entities: Iterable["DXFEntity"], drawing="doc"
+) -> "Code":
     code = _SourceCodeGenerator(doc=drawing)
     code.translate_entities(entities)
     return code.code
 
 
 class Code:
-    """ Source code container. """
+    """Source code container."""
 
     def __init__(self):
         self.code: List[str] = []
-        # global imports -> indention level 0:
+        # global imports -> indentation level 0:
         self.imports: Set[str] = set()
         # layer names as string:
         self.layers: Set[str] = set()
@@ -92,7 +141,7 @@ class Code:
         self.blocks: Set[str] = set()
 
     def code_str(self, indent: int = 0) -> str:
-        """ Returns the source code as a single string.
+        """Returns the source code as a single string.
 
         Args:
             indent: source code indentation count by spaces
@@ -101,13 +150,25 @@ class Code:
         lead_str = " " * indent
         return "\n".join(lead_str + line for line in self.code)
 
+    def black_code_str(self, line_length=88) -> str:
+        """Returns the source code as a single string formatted by `Black`_
+
+        Args:
+            line_length: max. source code line length
+
+        Raises:
+            ImportError: Black is not available
+
+        """
+        return black(self.code_str(), line_length)
+
     def __str__(self) -> str:
-        """ Returns the source code as a single string. """
+        """Returns the source code as a single string."""
 
         return self.code_str()
 
     def import_str(self, indent: int = 0) -> str:
-        """ Returns required imports as a single string.
+        """Returns required imports as a single string.
 
         Args:
             indent: source code indentation count by spaces
@@ -117,22 +178,22 @@ class Code:
         return "\n".join(lead_str + line for line in self.imports)
 
     def add_import(self, statement: str) -> None:
-        """ Add import statement, identical import statements are merged
+        """Add import statement, identical import statements are merged
         together.
         """
         self.imports.add(statement)
 
     def add_line(self, code: str, indent: int = 0) -> None:
-        """ Add a single source code line without line ending ``\\n``. """
+        """Add a single source code line without line ending ``\\n``."""
         self.code.append(" " * indent + code)
 
     def add_lines(self, code: Iterable[str], indent: int = 0) -> None:
-        """ Add multiple source code lines without line ending ``\\n``. """
+        """Add multiple source code lines without line ending ``\\n``."""
         for line in code:
             self.add_line(line, indent=indent)
 
     def merge(self, code: "Code", indent: int = 0) -> None:
-        """ Add another :class:`Code` object. """
+        """Add another :class:`Code` object."""
         # merge used resources
         self.imports.update(code.imports)
         self.layers.update(code.layers)
@@ -156,7 +217,7 @@ _PURGE_DXF_ATTRIBUTES = {
 
 
 def _purge_handles(attribs: dict) -> dict:
-    """ Purge handles from DXF attributes which will be invalid in a new
+    """Purge handles from DXF attributes which will be invalid in a new
     document, or which will be set automatically by adding an entity to a
     layout (paperspace).
 
@@ -175,7 +236,7 @@ def _fmt_mapping(mapping: Mapping, indent: int = 0) -> Iterable[str]:
         if isinstance(v, str):
             v = json.dumps(v)  # for correct escaping of quotes
         else:
-            v = str(v)  # format uses repr() for Vectors
+            v = str(v)  # format uses repr() for Vec3s
         yield fmt.format(k, v)
 
 
@@ -185,8 +246,9 @@ def _fmt_list(l: Iterable, indent: int = 0) -> Iterable[str]:
         yield fmt.format(str(v))
 
 
-def _fmt_api_call(func_call: str, args: Iterable[str],
-                  dxfattribs: dict) -> List[str]:
+def _fmt_api_call(
+    func_call: str, args: Iterable[str], dxfattribs: dict
+) -> List[str]:
     attributes = dict(dxfattribs)
     args = list(args) if args else []
 
@@ -206,7 +268,10 @@ def _fmt_api_call(func_call: str, args: Iterable[str],
     s.append("    dxfattribs={")
     s.extend(_fmt_mapping(attributes, indent=8))
     s.extend(
-        ["    },", ")", ]
+        [
+            "    },",
+            ")",
+        ]
     )
     return s
 
@@ -218,7 +283,7 @@ def _fmt_dxf_tags(tags: Iterable["DXFTag"], indent: int = 0):
         if isinstance(value, str):
             value = json.dumps(value)  # for correct escaping of quotes
         else:
-            value = str(value)  # format uses repr() for Vectors
+            value = str(value)  # format uses repr() for Vec3s
         yield fmt.format(code, value)
 
 
@@ -240,7 +305,7 @@ class _SourceCodeGenerator:
         self.code = Code()
 
     def translate_entity(self, entity: "DXFEntity") -> None:
-        """ Translates one DXF entity into Python source code. The generated
+        """Translates one DXF entity into Python source code. The generated
         source code is appended to the attribute `source_code`.
 
         Args:
@@ -251,15 +316,14 @@ class _SourceCodeGenerator:
         try:
             entity_translator = getattr(self, "_" + dxftype.lower())
         except AttributeError:
-            self.add_source_code_line(
-                f'# unsupported DXF entity "{dxftype}"'
-            )
+            self.add_source_code_line(f'# unsupported DXF entity "{dxftype}"')
         else:
             entity_translator(entity)
 
-    def translate_entities(self, entities: Iterable["DXFEntity"],
-                           ignore: Iterable[str] = None) -> None:
-        """ Translates multiple DXF entities into Python source code. The
+    def translate_entities(
+        self, entities: Iterable["DXFEntity"], ignore: Iterable[str] = None
+    ) -> None:
+        """Translates multiple DXF entities into Python source code. The
         generated source code is appended to the attribute `source_code`.
 
         Args:
@@ -275,7 +339,7 @@ class _SourceCodeGenerator:
                 self.translate_entity(entity)
 
     def add_used_resources(self, dxfattribs: Mapping) -> None:
-        """ Register used resources like layers, line types, text styles and
+        """Register used resources like layers, line types, text styles and
         dimension styles.
 
         Args:
@@ -298,32 +362,45 @@ class _SourceCodeGenerator:
         self.code.add_line(code)
 
     def add_source_code_lines(self, code: Iterable[str]) -> None:
+        assert not isinstance(code, str)
         self.code.add_lines(code)
 
-    def add_list_source_code(self, values: Iterable, prolog: str = "[",
-                             epilog: str = "]", indent: int = 0, ) -> None:
+    def add_list_source_code(
+        self,
+        values: Iterable,
+        prolog: str = "[",
+        epilog: str = "]",
+        indent: int = 0,
+    ) -> None:
         fmt_str = " " * indent + "{}"
         self.add_source_code_line(fmt_str.format(prolog))
         self.add_source_code_lines(_fmt_list(values, indent=4 + indent))
         self.add_source_code_line(fmt_str.format(epilog))
 
-    def add_dict_source_code(self, mapping: Mapping, prolog: str = "{",
-                             epilog: str = "}", indent: int = 0, ) -> None:
+    def add_dict_source_code(
+        self,
+        mapping: Mapping,
+        prolog: str = "{",
+        epilog: str = "}",
+        indent: int = 0,
+    ) -> None:
         fmt_str = " " * indent + "{}"
         self.add_source_code_line(fmt_str.format(prolog))
         self.add_source_code_lines(_fmt_mapping(mapping, indent=4 + indent))
         self.add_source_code_line(fmt_str.format(epilog))
 
-    def add_tags_source_code(self, tags: Tags, prolog="tags = Tags(",
-                             epilog=")", indent=4):
+    def add_tags_source_code(
+        self, tags: Tags, prolog="tags = Tags(", epilog=")", indent=4
+    ):
         fmt_str = " " * indent + "{}"
         self.add_source_code_line(fmt_str.format(prolog))
         self.add_source_code_lines(_fmt_dxf_tags(tags, indent=4 + indent))
         self.add_source_code_line(fmt_str.format(epilog))
 
-    def generic_api_call(self, dxftype: str, dxfattribs: dict,
-                         prefix: str = "e = ") -> Iterable[str]:
-        """ Returns the source code strings to create a DXF entity by a generic
+    def generic_api_call(
+        self, dxftype: str, dxfattribs: dict, prefix: str = "e = "
+    ) -> Iterable[str]:
+        """Returns the source code strings to create a DXF entity by a generic
         `new_entity()` call.
 
         Args:
@@ -341,13 +418,21 @@ class _SourceCodeGenerator:
         ]
         s.extend(_fmt_mapping(dxfattribs, indent=8))
         s.extend(
-            ["    },", ")", ]
+            [
+                "    },",
+                ")",
+            ]
         )
         return s
 
-    def api_call(self, api_call: str, args: Iterable[str], dxfattribs: dict,
-                 prefix: str = "e = ") -> Iterable[str]:
-        """ Returns the source code strings to create a DXF entity by the
+    def api_call(
+        self,
+        api_call: str,
+        args: Iterable[str],
+        dxfattribs: dict,
+        prefix: str = "e = ",
+    ) -> Iterable[str]:
+        """Returns the source code strings to create a DXF entity by the
         specialised API call.
 
         Args:
@@ -362,7 +447,7 @@ class _SourceCodeGenerator:
         return _fmt_api_call(func_call, args, dxfattribs)
 
     def new_table_entry(self, dxftype: str, dxfattribs: dict) -> Iterable[str]:
-        """ Returns the source code strings to create a new table entity by
+        """Returns the source code strings to create a new table entity by
         ezdxf.
 
         Args:
@@ -381,7 +466,10 @@ class _SourceCodeGenerator:
         ]
         s.extend(_fmt_mapping(dxfattribs, indent=12))
         s.extend(
-            ["        },", "    )", ]
+            [
+                "        },",
+                "    )",
+            ]
         )
         return s
 
@@ -467,7 +555,7 @@ class _SourceCodeGenerator:
         )
         self.add_source_code_line(
             '# Set valid handles or remove attributes ending with "_handle", '
-            'otherwise the DXF file is invalid for AutoCAD'
+            "otherwise the DXF file is invalid for AutoCAD"
         )
 
     # complex graphical types
@@ -490,7 +578,7 @@ class _SourceCodeGenerator:
                         "ATTRIB", attrib.dxfattribs(), prefix="a = "
                     )
                 )
-                self.add_source_code_lines("e.attribs.append(a)")
+                self.add_source_code_line("e.attribs.append(a)")
 
     def _mtext(self, entity: "MText") -> None:
         self.add_source_code_lines(
@@ -593,7 +681,7 @@ class _SourceCodeGenerator:
         )
         if len(entity.boundary_path):
             self.add_list_source_code(
-                (v[:2] for v in entity.boundary_path),  # just x, y axis
+                (v for v in entity.boundary_path),  # just x, y axis
                 prolog="e.set_boundary_path([",
                 epilog="])",
             )
@@ -608,7 +696,7 @@ class _SourceCodeGenerator:
         )
         if len(entity.boundary_path):
             self.add_list_source_code(
-                (v[:2] for v in entity.boundary_path),  # just x, y axis
+                (v for v in entity.boundary_path),  # just x, y axis
                 prolog="e.set_boundary_path([",
                 epilog="])",
             )
@@ -641,12 +729,26 @@ class _SourceCodeGenerator:
             )
 
     def _hatch(self, entity: "Hatch"):
-        add_line = self.add_source_code_line
         dxfattribs = entity.dxfattribs()
         dxfattribs["associative"] = 0  # associative hatch not supported
         self.add_source_code_lines(
             self.api_call("add_hatch(", ["color"], dxfattribs)
         )
+        self._polygon(entity)
+
+    def _mpolygon(self, entity: "MPolygon"):
+        dxfattribs = entity.dxfattribs()
+        self.add_source_code_lines(
+            self.api_call("add_mpolygon(", ["color"], dxfattribs)
+        )
+        if entity.dxf.solid_fill:
+            self.add_source_code_line(
+                f"e.set_solid_fill(color={entity.dxf.fill_color})\n"
+            )
+        self._polygon(entity)
+
+    def _polygon(self, entity: "DXFPolygon"):
+        add_line = self.add_source_code_line
         if len(entity.seeds):
             add_line(f"e.set_seed_points({entity.seeds})")
         if entity.pattern:
@@ -655,9 +757,10 @@ class _SourceCodeGenerator:
                 prolog="e.set_pattern_definition([",
                 epilog="])",
             )
+            self.add_source_code_line("e.dxf.solid_fill = 0")
         arg = "    {}={},"
 
-        if entity.has_gradient_data:
+        if entity.gradient is not None:
             g = entity.gradient
             add_line("e.set_gradient(")
             add_line(arg.format("color1", str(g.color1)))
@@ -668,7 +771,7 @@ class _SourceCodeGenerator:
             add_line(arg.format("name", json.dumps(g.name)))
             add_line(")")
         for count, path in enumerate(entity.paths, start=1):
-            if path.PATH_TYPE == "PolylinePath":
+            if path.type == BoundaryPathType.POLYLINE:
                 add_line("# {}. polyline path".format(count))
                 self.add_list_source_code(
                     path.vertices,
@@ -686,43 +789,43 @@ class _SourceCodeGenerator:
                     f"ep = e.paths.add_edge_path(flags={path.path_type_flags})"
                 )
                 for edge in path.edges:
-                    if edge.EDGE_TYPE == "LineEdge":
-                        add_line(
-                            f"ep.add_line({edge.start[:2]}, {str(edge.end[:2])})"
-                        )
-                    elif edge.EDGE_TYPE == "ArcEdge":
+                    if edge.type == EdgeType.LINE:
+                        add_line(f"ep.add_line({edge.start}, {str(edge.end)})")
+                    elif edge.type == EdgeType.ARC:
+                        # Start- and end angles are always stored in ccw
+                        # orientation:
                         add_line("ep.add_arc(")
-                        add_line(arg.format("center", str(edge.center[:2])))
+                        add_line(arg.format("center", str(edge.center)))
                         add_line(arg.format("radius", edge.radius))
                         add_line(arg.format("start_angle", edge.start_angle))
                         add_line(arg.format("end_angle", edge.end_angle))
                         add_line(arg.format("ccw", edge.ccw))
                         add_line(")")
-                    elif edge.EDGE_TYPE == "EllipseEdge":
+                    elif edge.type == EdgeType.ELLIPSE:
+                        # Start- and end params are always stored in ccw
+                        # orientation:
                         add_line("ep.add_ellipse(")
-                        add_line(arg.format("center", str(edge.center[:2])))
-                        add_line(
-                            arg.format("major_axis", str(edge.major_axis[:2]))
-                        )
+                        add_line(arg.format("center", str(edge.center)))
+                        add_line(arg.format("major_axis", str(edge.major_axis)))
                         add_line(arg.format("ratio", edge.ratio))
                         add_line(arg.format("start_angle", edge.start_angle))
                         add_line(arg.format("end_angle", edge.end_angle))
                         add_line(arg.format("ccw", edge.ccw))
                         add_line(")")
-                    elif edge.EDGE_TYPE == "SplineEdge":
+                    elif edge.type == EdgeType.SPLINE:
                         add_line("ep.add_spline(")
                         if edge.fit_points:
                             add_line(
                                 arg.format(
                                     "fit_points",
-                                    str([fp[:2] for fp in edge.fit_points]),
+                                    str([fp for fp in edge.fit_points]),
                                 )
                             )
                         if edge.control_points:
                             add_line(
                                 arg.format(
                                     "control_points",
-                                    str([cp[:2] for cp in edge.control_points]),
+                                    str([cp for cp in edge.control_points]),
                                 )
                             )
                         if edge.knot_values:
