@@ -1,9 +1,9 @@
 # Purpose: table, consisting of basic R12 entities
 # Created: 18.03.2010, 2018 adapted for ezdxf
-# Copyright (c) 2010-2018, Manfred Moitzi
+# Copyright (c) 2010-2021, Manfred Moitzi
 # License: MIT License
 """
-Table object like a HTML-Table, as composite pf basic DXF R12 entities.
+Table object like a HTML-Table, as composite of basic DXF R12 entities.
 
 Cells can contain MText or BLOCK references, or you can create your own
 cell type by extending the CustomCell() class.
@@ -15,20 +15,26 @@ BlockCells contains block references (INSERT) created from a block
 definition (BLOCK), if the block definition contains attribute definitions
 (ATTDEF), attribs will be added to the block reference (ATTRIB).
 
+This add-on exist only for porting 'dxfwrite' projects to 'ezdxf'.
+
+This add-on works completely different than the ACAD_TABLE entity, they have
+nothing in common!
+
 """
 
 from copy import deepcopy
 from abc import abstractmethod
 
 from ezdxf.lldxf import const
+from ezdxf.enums import MAP_FLAGS_TO_STRING_ALIGN, MAP_STRING_ALIGN_TO_FLAGS
 from .mtext import MText
 
-DEFAULT_TABLE_BGLAYER = 'TABLEBACKGROUND'
-DEFAULT_TABLE_FGLAYER = 'TABLECONTENT'
-DEFAULT_TABLE_GRIDLAYER = 'TABLEGRID'
+DEFAULT_TABLE_BGLAYER = "TABLEBACKGROUND"
+DEFAULT_TABLE_FGLAYER = "TABLECONTENT"
+DEFAULT_TABLE_GRIDLAYER = "TABLEGRID"
 DEFAULT_TABLE_HEIGHT = 1.0
 DEFAULT_TABLE_WIDTH = 2.5
-DEFAULT_TEXTSTYLE = 'STANDARD'
+DEFAULT_TEXTSTYLE = "STANDARD"
 DEFAULT_CELL_TEXT_HEIGHT = 0.7
 DEFAULT_CELL_LINESPACING = 1.5
 DEFAULT_CELL_XSCALE = 1.0
@@ -52,7 +58,8 @@ class Table(object):
     The table object contains the table data cells.
 
     """
-    name = 'TABLE'
+
+    name = "TABLE"
 
     def __init__(self, insert, nrows, ncols, default_grid=True):
         """
@@ -72,9 +79,9 @@ class Table(object):
         self.bglayer = DEFAULT_TABLE_BGLAYER
         self.fglayer = DEFAULT_TABLE_FGLAYER
         self.gridlayer = DEFAULT_TABLE_GRIDLAYER
-        self.styles = {'default': Style.get_default_cell_style()}
+        self.styles = {"default": Style.get_default_cell_style()}
         if not default_grid:
-            default_style = self.get_cell_style('default')
+            default_style = self.get_cell_style("default")
             default_style.set_border_status(False, False, False, False)
 
         self._cells = {}  # data cells
@@ -103,7 +110,7 @@ class Table(object):
         """
         self.row_heights[row] = float(value)
 
-    def text_cell(self, row, col, text, span=(1, 1), style='default'):
+    def text_cell(self, row, col, text, span=(1, 1), style="default"):
         """
         Create a new text cell at position (row, col), with 'text' as
         content, text can be a multi-line text, use ``'\\n'`` as line
@@ -116,7 +123,9 @@ class Table(object):
         cell = TextCell(self, text, style=style, span=span)
         return self.set_cell(row, col, cell)
 
-    def block_cell(self, row, col, blockdef, span=(1, 1), attribs=None, style='default'):
+    def block_cell(
+        self, row, col, blockdef, span=(1, 1), attribs=None, style="default"
+    ):
         """
         Create a new block cell at position (row, col).
 
@@ -133,7 +142,9 @@ class Table(object):
         """
         if attribs is None:
             attribs = {}
-        cell = BlockCell(self, blockdef, style=style, attribs=attribs, span=span)
+        cell = BlockCell(
+            self, blockdef, style=style, attribs=attribs, span=span
+        )
         return self.set_cell(row, col, cell)
 
     def set_cell(self, row, col, cell):
@@ -157,17 +168,15 @@ class Table(object):
     def validate_index(self, row, col):
         row = int(row)
         col = int(col)
-        if row < 0 or row >= self.nrows or \
-           col < 0 or col >= self.ncols:
-            raise IndexError('cell index out of range')
+        if row < 0 or row >= self.nrows or col < 0 or col >= self.ncols:
+            raise IndexError("cell index out of range")
         return row, col
 
-    def frame(self, row, col, width=1, height=1, style='default'):
+    def frame(self, row, col, width=1, height=1, style="default"):
         """
         Create a Frame object which frames the cell area starting at(row, col) covering 'width' columns and 'height' rows.
         """
-        frame = Frame(self, pos=(row, col), span=(height, width),
-                      style=style)
+        frame = Frame(self, pos=(row, col), span=(height, width), style=style)
         self.frames.append(frame)
         return frame
 
@@ -178,22 +187,24 @@ class Table(object):
         Args:
             kwargs: see Style.get_default_cell_style()
         """
-        style = deepcopy(self.get_cell_style('default'))
+        style = deepcopy(self.get_cell_style("default"))
         style.update(kwargs)
-        if 'align' in kwargs:
-            align = kwargs.get('align')
-            halign, valign = const.TEXT_ALIGN_FLAGS.get(align)
-            style['halign'] = halign
-            style['valign'] = valign
+        if "align" in kwargs:
+            align = kwargs.get("align")
+            halign, valign = MAP_STRING_ALIGN_TO_FLAGS.get(align)
+            style["halign"] = halign
+            style["valign"] = valign
         else:
-            halign = kwargs.get('halign')
-            valign = kwargs.get('valign')
-            style['align'] = const.TEXT_ALIGNMENT_BY_FLAGS.get(halign, valign)
+            halign = kwargs.get("halign")
+            valign = kwargs.get("valign")
+            style["align"] = MAP_FLAGS_TO_STRING_ALIGN.get(halign, valign)
 
         self.styles[name] = style
         return style
 
-    def new_border_style(self, color=const.BYLAYER, status=True, priority=100, linetype="BYLAYER"):
+    def new_border_style(
+        self, color=const.BYLAYER, status=True, priority=100, linetype="BYLAYER"
+    ):
         """
         Create a new border style.
 
@@ -204,10 +215,10 @@ class Table(object):
             priority: drawing priority - higher values covers lower values
         """
         border_style = Style.get_default_border_style()
-        border_style['color'] = color
-        border_style['linetype'] = linetype
-        border_style['status'] = status
-        border_style['priority'] = priority
+        border_style["color"] = color
+        border_style["linetype"] = linetype
+        border_style["status"] = status
+        border_style["priority"] = priority
         return border_style
 
     def get_cell_style(self, name):
@@ -222,7 +233,9 @@ class Table(object):
 
         Returns: a generator which yields all visible cells as tuples: (row , col, cell)
         """
-        return ((row, col, self.get_cell(row, col)) for row, col in visibility_map)
+        return (
+            (row, col, self.get_cell(row, col)) for row, col in visibility_map
+        )
 
     def render(self, layout, insert=None):
         """
@@ -245,6 +258,7 @@ class VisibilityMap(object):
     """
     Stores the visibility of the table cells.
     """
+
     def __init__(self, table):
         """
         Create the visibility map for table.
@@ -275,7 +289,7 @@ class VisibilityMap(object):
             for rowx in range(nrows):
                 for colx in range(ncols):
                     # switch all cells in span range to invisible
-                    self.hide(row+rowx, col+colx)
+                    self.hide(row + rowx, col + colx)
         # switch content cell visible
         self.show(row, col)
 
@@ -312,85 +326,98 @@ class VisibilityMap(object):
         """
         Iterate over all visible cells.
         """
-        return ((row, col) for (row, col) in self.iter_all_cells() if self.is_visible_cell(row, col))
+        return (
+            (row, col)
+            for (row, col) in self.iter_all_cells()
+            if self.is_visible_cell(row, col)
+        )
 
 
 class Style(dict):
     """
     Cell style object.
     """
+
     @staticmethod
     def get_default_cell_style():
-        return Style({
-            # textstyle is ignored by block cells
-            'textstyle': 'STANDARD',
-            # text height in drawing units, ignored by block cells
-            'textheight': DEFAULT_CELL_TEXT_HEIGHT,
-            # line spacing in percent = <textheight>*<linespacing>, ignored by block cells
-            'linespacing': DEFAULT_CELL_LINESPACING,
-            # text stretch or block reference x-axis scaling factor
-            'xscale': DEFAULT_CELL_XSCALE,
-            # block reference y-axis scaling factor, ignored by text cells
-            'yscale': DEFAULT_CELL_YSCALE,
-            # dxf color index, ignored by block cells
-            'textcolor': DEFAULT_CELL_TEXTCOLOR,
-            # text or block rotation in degrees
-            'rotation': 0.,
-            # Letters are stacked top-to-bottom, but not rotated
-            'stacked': False,
-            # simple combined align parameter, like 'TOP_CENTER', see also MText.VALID_ALIGN
-            'align': 'TOP_CENTER',  # higher priority than 'haling' and 'valign'
-            # horizontal alignment (const.LEFT, const.CENTER, const.RIGHT)
-            'halign': const.CENTER,
-            # vertical alignment (const.TOP, const.MIDDLE, const.BOTTOM)
-            'valign': const.TOP,
-            # left and right margin in drawing units
-            'hmargin': DEFAULT_CELL_HMARGIN,
-            # top and bottom margin
-            'vmargin': DEFAULT_CELL_VMARGIN,
-            # background color, dxf color index, ignored by block cells
-            'bgcolor': DEFAULT_CELL_BG_COLOR,
-            # left border style
-            'left': Style.get_default_border_style(),
-            # top border style
-            'top': Style.get_default_border_style(),
-            # right border style
-            'right': Style.get_default_border_style(),
-            # bottom border style
-            'bottom': Style.get_default_border_style(),
-        })
+        return Style(
+            {
+                # textstyle is ignored by block cells
+                "textstyle": "STANDARD",
+                # text height in drawing units, ignored by block cells
+                "textheight": DEFAULT_CELL_TEXT_HEIGHT,
+                # line spacing in percent = <textheight>*<linespacing>, ignored by block cells
+                "linespacing": DEFAULT_CELL_LINESPACING,
+                # text stretch or block reference x-axis scaling factor
+                "xscale": DEFAULT_CELL_XSCALE,
+                # block reference y-axis scaling factor, ignored by text cells
+                "yscale": DEFAULT_CELL_YSCALE,
+                # dxf color index, ignored by block cells
+                "textcolor": DEFAULT_CELL_TEXTCOLOR,
+                # text or block rotation in degrees
+                "rotation": 0.0,
+                # Letters are stacked top-to-bottom, but not rotated
+                "stacked": False,
+                # simple combined align parameter, like 'TOP_CENTER', see also MText.VALID_ALIGN
+                "align": "TOP_CENTER",  # higher priority than 'haling' and 'valign'
+                # horizontal alignment (const.LEFT, const.CENTER, const.RIGHT)
+                "halign": const.CENTER,
+                # vertical alignment (const.TOP, const.MIDDLE, const.BOTTOM)
+                "valign": const.TOP,
+                # left and right margin in drawing units
+                "hmargin": DEFAULT_CELL_HMARGIN,
+                # top and bottom margin
+                "vmargin": DEFAULT_CELL_VMARGIN,
+                # background color, dxf color index, ignored by block cells
+                "bgcolor": DEFAULT_CELL_BG_COLOR,
+                # left border style
+                "left": Style.get_default_border_style(),
+                # top border style
+                "top": Style.get_default_border_style(),
+                # right border style
+                "right": Style.get_default_border_style(),
+                # bottom border style
+                "bottom": Style.get_default_border_style(),
+            }
+        )
 
     @staticmethod
     def get_default_border_style():
         return {
             # border status, True for visible, False for hidden
-            'status': DEFAULT_BORDER_STATUS,
+            "status": DEFAULT_BORDER_STATUS,
             # dxf color index
-            'color': DEFAULT_BORDER_COLOR,
+            "color": DEFAULT_BORDER_COLOR,
             # linetype name, BYLAYER if None
-            'linetype': DEFAULT_BORDER_LINETYPE,
+            "linetype": DEFAULT_BORDER_LINETYPE,
             # drawing priority, higher values cover lower values
-            'priority': DEFAULT_BORDER_PRIORITY,
+            "priority": DEFAULT_BORDER_PRIORITY,
         }
 
     def set_border_status(self, left=True, right=True, top=True, bottom=True):
         """
         Set status of all cell borders at once.
         """
-        for border, status in (('left', left),
-                               ('right', right),
-                               ('top', top),
-                               ('bottom', bottom)):
-            self[border]['status'] = status
+        for border, status in (
+            ("left", left),
+            ("right", right),
+            ("top", top),
+            ("bottom", bottom),
+        ):
+            self[border]["status"] = status
 
-    def set_border_style(self, style, left=True, right=True, top=True, bottom=True):
+    def set_border_style(
+        self, style, left=True, right=True, top=True, bottom=True
+    ):
         """
         Set border styles of all cell borders at once.
         """
-        for border, status in (('left', left),
-                               ('right', right),
-                               ('top', top),
-                               ('bottom', bottom)):
+        for border, status in (
+            ("left", left),
+            ("right", right),
+            ("top", top),
+            ("bottom", bottom),
+        ):
             if status:
                 self[border] = style
 
@@ -399,6 +426,7 @@ class Grid(object):
     """
     Grid contains the graphical representation of the table.
     """
+
     def __init__(self, table):
         self.table = table
         # contains the x-axis coords of the grid lines between the data columns.
@@ -410,13 +438,15 @@ class Grid(object):
         # above row, col, and row-indices are [0 .. nrows+1], nrows+1 for the
         # grid line below the last row; list contains only the border style with
         # the highest priority.
-        self._hborders = None # created in _init_borders
+        self._hborders = None  # created in _init_borders
         # same as _hborders but for the vertical borders,
         # col-indices are [0 .. ncols+1], ncols+1 for the last grid line right
         # of the last column
-        self._vborders = None # created in _init_borders
+        self._vborders = None  # created in _init_borders
         # border style to delete borders inside of merged cells
-        self.noborder = dict(status=False, priority=999, linetype="BYLAYER", color=0)
+        self.noborder = dict(
+            status=False, priority=999, linetype="BYLAYER", color=0
+        )
 
     def _init_borders(self, hborder, vborder):
         """
@@ -427,7 +457,7 @@ class Grid(object):
         # exact values are:
         # hborder_count = ncols * (nrows+1), hindex = ncols * <row> + <col>
         # vborder_count = nrows * (ncols+1), vindex = (ncols+1) * <row> + <col>
-        border_count = (self.table.nrows+1) * (self.table.ncols+1)
+        border_count = (self.table.nrows + 1) * (self.table.ncols + 1)
         self._hborders = [hborder] * border_count
         self._vborders = [vborder] * border_count
 
@@ -435,7 +465,7 @@ class Grid(object):
         """
         Calculate linear index for border arrays _hborders and _vborders.
         """
-        return row * (self.table.ncols+1) + col
+        return row * (self.table.ncols + 1) + col
 
     def set_hborder(self, row, col, border_style):
         """
@@ -455,7 +485,7 @@ class Grid(object):
         """
         border_index = self._border_index(row, col)
         actual_borderstyle = borders[border_index]
-        if border_style['priority'] >= actual_borderstyle['priority']:
+        if border_style["priority"] >= actual_borderstyle["priority"]:
             borders[border_index] = border_style
 
     def get_hborder(self, row, col):
@@ -478,7 +508,7 @@ class Grid(object):
         """
         return borders[self._border_index(row, col)]
 
-    def _sum_fields(self, start_value, fields, append, sign=1.):
+    def _sum_fields(self, start_value, fields, append, sign=1.0):
         """
         Adds step-by-step the fields-values, starting with <start_value>,
         and appends the resulting values to an other object with the
@@ -491,34 +521,28 @@ class Grid(object):
             append(position)
 
     def _calc_col_pos(self):
-        """ Calculate the x-axis coords of the grid lines between the columns.
-        """
+        """Calculate the x-axis coords of the grid lines between the columns."""
         col_pos = []
         start_x = self.table.insert[0]
-        self._sum_fields(start_x,
-                         self.table.col_widths,
-                         col_pos.append)
+        self._sum_fields(start_x, self.table.col_widths, col_pos.append)
         return col_pos
 
     def _calc_row_pos(self):
-        """ Calculate the y-axis coords of the grid lines between the rows.
-        """
+        """Calculate the y-axis coords of the grid lines between the rows."""
         row_pos = []
         start_y = self.table.insert[1]
-        self._sum_fields(start_y,
-                         self.table.row_heights,
-                         row_pos.append, -1.)
+        self._sum_fields(start_y, self.table.row_heights, row_pos.append, -1.0)
         return row_pos
 
     def cell_coords(self, row, col, span):
-        """ Get the coordinates of the cell <row>,<col> as absolute drawing units.
+        """Get the coordinates of the cell <row>,<col> as absolute drawing units.
 
         :return: a tuple (left, right, top, bottom)
         """
         top = self.row_pos[row]
-        bottom = self.row_pos[row+span[0]]
+        bottom = self.row_pos[row + span[0]]
         left = self.col_pos[col]
-        right = self.col_pos[col+span[1]]
+        right = self.col_pos[col + span[1]]
         return left, right, top, bottom
 
     def render_cell_background(self, layout, row, col, cell):
@@ -526,7 +550,7 @@ class Grid(object):
         Render the cell background for <row>, <col> as SOLID entity.
         """
         style = cell.style
-        if style['bgcolor'] is None:
+        if style["bgcolor"] is None:
             return
         # get cell coords in absolute drawing units
         left, right, top, bottom = self.cell_coords(row, col, cell.span)
@@ -537,9 +561,10 @@ class Grid(object):
         layout.add_solid(
             points=(ltop, lbot, rtop, rbot),
             dxfattribs={
-                'color': style['bgcolor'],
-                'layer': self.table.bglayer,
-            })
+                "color": style["bgcolor"],
+                "layer": self.table.bglayer,
+            },
+        )
 
     def render_cell_content(self, layout, row, col, cell):
         """
@@ -554,9 +579,9 @@ class Grid(object):
         Render all grid lines into layout object.
         """
         # Init borders with default_style top- and left border.
-        default_style = self.table.get_cell_style('default')
-        hborder = default_style['top']
-        vborder = default_style['left']
+        default_style = self.table.get_cell_style("default")
+        hborder = default_style["top"]
+        vborder = default_style["left"]
         self._init_borders(hborder, vborder)
         self._set_frames(self.table.frames)
         self._set_borders(self.table.iter_visible_cells(visibility_map))
@@ -570,23 +595,28 @@ class Grid(object):
             bottom_row = row + cell.span[0]
             right_col = col + cell.span[1]
             self._set_rect_borders(row, bottom_row, col, right_col, cell.style)
-            self._set_inner_borders(row, bottom_row, col, right_col,
-                                    self.noborder)
+            self._set_inner_borders(
+                row, bottom_row, col, right_col, self.noborder
+            )
 
-    def _set_inner_borders(self, top_row, bottom_row, left_col, right_col, border_style):
+    def _set_inner_borders(
+        self, top_row, bottom_row, left_col, right_col, border_style
+    ):
         """
         Set <border_style> to the inner borders of the rectangle <top_row...
         """
         if bottom_row - top_row > 1:
             for col in range(left_col, right_col):
-                for row in range(top_row+1, bottom_row):
+                for row in range(top_row + 1, bottom_row):
                     self.set_hborder(row, col, border_style)
         if right_col - left_col > 1:
             for row in range(top_row, bottom_row):
-                for col in range(left_col+1, right_col):
+                for col in range(left_col + 1, right_col):
                     self.set_vborder(row, col, border_style)
 
-    def _set_rect_borders(self, top_row, bottom_row, left_col, right_col, style):
+    def _set_rect_borders(
+        self, top_row, bottom_row, left_col, right_col, style
+    ):
         """
         Set border <style> to the rectangle <top_row><bottom_row...
 
@@ -594,11 +624,11 @@ class Grid(object):
         comments for self._hborders and self._vborders.
         """
         for col in range(left_col, right_col):
-            self.set_hborder(top_row, col, style['top'])
-            self.set_hborder(bottom_row, col, style['bottom'])
+            self.set_hborder(top_row, col, style["top"])
+            self.set_hborder(bottom_row, col, style["bottom"])
         for row in range(top_row, bottom_row):
-            self.set_vborder(row, left_col, style['left'])
-            self.set_vborder(row, right_col, style['right'])
+            self.set_vborder(row, left_col, style["left"])
+            self.set_vborder(row, right_col, style["right"])
 
     def _set_frames(self, frames):
         """
@@ -609,36 +639,39 @@ class Grid(object):
             left_col = frame.pos[1]
             bottom_row = top_row + frame.span[0]
             right_col = left_col + frame.span[1]
-            self._set_rect_borders(top_row, bottom_row, left_col, right_col, frame.style)
+            self._set_rect_borders(
+                top_row, bottom_row, left_col, right_col, frame.style
+            )
 
     def _render_borders(self, layout, table):
         """
         Render the grid lines as LINE entities into layout object.
         """
+
         def render_line(start, end, style):
             """
             Render the LINE entity into layout object.
             """
-            if style['status']:
+            if style["status"]:
                 layout.add_line(
                     start=start,
                     end=end,
                     dxfattribs={
-                        'layer': layer,
-                        'color': style['color'],
-                        'linetype': style['linetype']
-                    }
+                        "layer": layer,
+                        "color": style["color"],
+                        "linetype": style["linetype"],
+                    },
                 )
 
         def render_hborders():
             """
             Draw the horizontal grid lines.
             """
-            for row in range(table.nrows+1):
+            for row in range(table.nrows + 1):
                 yrow = self.row_pos[row]
                 for col in range(table.ncols):
                     xleft = self.col_pos[col]
-                    xright = self.col_pos[col+1]
+                    xright = self.col_pos[col + 1]
                     style = self.get_hborder(row, col)
                     render_line((xleft, yrow), (xright, yrow), style)
 
@@ -646,11 +679,11 @@ class Grid(object):
             """
             Draw the vertical grid lines.
             """
-            for col in range(table.ncols+1):
+            for col in range(table.ncols + 1):
                 xcol = self.col_pos[col]
                 for row in range(table.nrows):
                     ytop = self.row_pos[row]
-                    ybottom = self.row_pos[row+1]
+                    ybottom = self.row_pos[row + 1]
                     style = self.get_vborder(row, col)
                     render_line((xcol, ytop), (xcol, ybottom), style)
 
@@ -663,7 +696,8 @@ class Frame(object):
     """
     Represent a rectangle cell area enclosed by border lines.
     """
-    def __init__(self, table, pos=(0, 0), span=(1, 1), style='default'):
+
+    def __init__(self, table, pos=(0, 0), span=(1, 1), style="default"):
         """
         Constructor
 
@@ -687,7 +721,8 @@ class Cell(object):
     """
     Cell represents the table cell data.
     """
-    def __init__(self, table, style='default', span=(1, 1)):
+
+    def __init__(self, table, style="default", span=(1, 1)):
         """
         Constructor
 
@@ -731,12 +766,14 @@ class Cell(object):
         """
         Reduces the cell-coords about the hmargin and the vmargin values.
         """
-        hmargin = self.style['hmargin']
-        vmargin = self.style['vmargin']
-        return (coords[0] + hmargin,  # left
-                coords[1] - hmargin,  # right
-                coords[2] - vmargin,  # top
-                coords[3] + vmargin)  # bottom
+        hmargin = self.style["hmargin"]
+        vmargin = self.style["vmargin"]
+        return (
+            coords[0] + hmargin,  # left
+            coords[1] - hmargin,  # right
+            coords[2] - vmargin,  # top
+            coords[3] + vmargin,
+        )  # bottom
 
 
 CustomCell = Cell
@@ -746,7 +783,8 @@ class TextCell(Cell):
     """
     Represents a multi line text. Text lines are separated by '\n'.
     """
-    def __init__(self, table,  text, style='default', span=(1, 1)):
+
+    def __init__(self, table, text, style="default", span=(1, 1)):
         """
         Constructor
 
@@ -775,26 +813,26 @@ class TextCell(Cell):
 
         left, right, top, bottom = self.get_workspace_coords(coords)
         style = self.style
-        halign = style['halign']
-        valign = style['valign']
-        rotated = self.style['rotation']
+        halign = style["halign"]
+        valign = style["valign"]
+        rotated = self.style["rotation"]
         text = self.text
-        if style['stacked']:
-            rotated = 0.
-            text = '\n'.join((char for char in self.text.replace('\n', ' ')))
-        xpos = (left, float(left+right)/2., right)[halign]
-        ypos = (bottom, float(bottom+top)/2., top)[valign-1]
+        if style["stacked"]:
+            rotated = 0.0
+            text = "\n".join((char for char in self.text.replace("\n", " ")))
+        xpos = (left, float(left + right) / 2.0, right)[halign]
+        ypos = (bottom, float(bottom + top) / 2.0, top)[valign - 1]
         mtext = MText(  # using dxfwrite MText() composite, because it works
             text,
             (xpos, ypos),
-            linespacing=self.style['linespacing'],
-            style=self.style['textstyle'],
-            height=self.style['textheight'],
+            linespacing=self.style["linespacing"],
+            style=self.style["textstyle"],
+            height=self.style["textheight"],
             rotation=rotated,
-            xscale=self.style['xscale'],
+            xscale=self.style["xscale"],
             halign=halign,
             valign=valign,
-            color=self.style['textcolor'],
+            color=self.style["textcolor"],
             layer=layer,
         )
         mtext.render(layout)
@@ -804,7 +842,10 @@ class BlockCell(Cell):
     """
     Cell that contains a block reference.
     """
-    def __init__(self, table, blockdef, style='default', attribs=None, span=(1, 1)):
+
+    def __init__(
+        self, table, blockdef, style="default", attribs=None, span=(1, 1)
+    ):
         """
         Constructor
 
@@ -834,18 +875,18 @@ class BlockCell(Cell):
         """
         left, right, top, bottom = self.get_workspace_coords(coords)
         style = self.style
-        halign = style['halign']
-        valign = style['valign']
-        xpos = (left, float(left + right) / 2., right)[halign]
-        ypos = (bottom, float(bottom + top) / 2., top)[valign-1]
+        halign = style["halign"]
+        valign = style["valign"]
+        xpos = (left, float(left + right) / 2.0, right)[halign]
+        ypos = (bottom, float(bottom + top) / 2.0, top)[valign - 1]
         layout.add_auto_blockref(
             name=self.blockdef.name,
             insert=(xpos, ypos),
             values=self.attribs,
             dxfattribs={
-                'xscale': style['xscale'],
-                'yscale': style['yscale'],
-                'rotation': style['rotation'],
-                'layer': layer,
-            }
+                "xscale": style["xscale"],
+                "yscale": style["yscale"],
+                "rotation": style["rotation"],
+                "layer": layer,
+            },
         )
