@@ -1,84 +1,155 @@
-# Copyright (c) 2019-2020 Manfred Moitzi
+# Copyright (c) 2019-2022 Manfred Moitzi
 # License: MIT License
-# Created 2019-02-16
+from __future__ import annotations
 from typing import (
-    TYPE_CHECKING, Iterable, cast, Tuple, Union, Optional,
-    Callable, Dict,
+    TYPE_CHECKING,
+    Iterable,
+    cast,
+    Tuple,
+    Union,
+    Optional,
+    Callable,
+    Dict,
+    List,
 )
 import math
-import warnings
 from ezdxf.lldxf import validator
 from ezdxf.lldxf.attributes import (
-    DXFAttr, DXFAttributes, DefSubclass, XType, RETURN_DEFAULT,
+    DXFAttr,
+    DXFAttributes,
+    DefSubclass,
+    XType,
+    RETURN_DEFAULT,
+    group_code_mapping,
+    merge_group_code_mappings,
 )
 from ezdxf.lldxf.const import (
-    DXF12, SUBCLASS_MARKER, DXFValueError, DXFKeyError, DXFStructureError,
+    DXF12,
+    SUBCLASS_MARKER,
+    DXFValueError,
+    DXFKeyError,
+    DXFStructureError,
 )
 from ezdxf.math import (
-    Vector, X_AXIS, Y_AXIS, Z_AXIS, Matrix44, OCS, UCS, NULLVEC,
+    Vec3,
+    UVec,
+    X_AXIS,
+    Y_AXIS,
+    Z_AXIS,
+    Matrix44,
+    OCS,
+    UCS,
+    NULLVEC,
 )
 from ezdxf.math.transformtools import OCSTransform, InsertTransformationError
 from ezdxf.explode import (
-    explode_block_reference, virtual_block_reference_entities,
+    explode_block_reference,
+    virtual_block_reference_entities,
 )
 from ezdxf.entities import factory
 from ezdxf.query import EntityQuery
 from ezdxf.audit import AuditError
 from .dxfentity import base_class, SubclassProcessor
-from .dxfgfx import DXFGraphic, acdb_entity
+from .dxfgfx import (
+    DXFGraphic,
+    acdb_entity,
+    elevation_to_z_axis,
+    acdb_entity_group_codes,
+)
 from .subentity import LinkedEntities
 from .attrib import Attrib
 
 if TYPE_CHECKING:
     from ezdxf.eztypes import (
-        TagWriter, Vertex, DXFNamespace, AttDef, BlockLayout, BaseLayout,
+        TagWriter,
+        DXFNamespace,
+        AttDef,
+        BlockLayout,
+        BaseLayout,
         Auditor,
+        BlockRecord,
     )
 
-__all__ = ['Insert']
+__all__ = ["Insert"]
+
+ABS_TOL = 1e-9
+
+# DXF files as XREF:
+# The INSERT entity is used to attach XREFS.
+# The model space is the block content, if the whole document is used as
+# BLOCK (XREF), but this is only supported for the DWG format.
+# AutoCAD does not support DXF files as XREFS, they are ignored, but the DXF
+# file is valid! BricsCAD shows DXF files as XREFS, but does not allow to attach
+# DXF files as XREFS by the application itself.
 
 # Multi-INSERT has subclass id AcDbMInsertBlock
-acdb_block_reference = DefSubclass('AcDbBlockReference', {
-    'attribs_follow': DXFAttr(66, default=0, optional=True),
-    'name': DXFAttr(2, validator=validator.is_valid_block_name),
-    'insert': DXFAttr(10, xtype=XType.any_point),
-    'xscale': DXFAttr(
-        41, default=1, optional=True,
-        validator=validator.is_not_zero,
-        fixer=RETURN_DEFAULT,
-    ),
-    'yscale': DXFAttr(
-        42, default=1, optional=True,
-        validator=validator.is_not_zero,
-        fixer=RETURN_DEFAULT,
-    ),
-    'zscale': DXFAttr(
-        43, default=1, optional=True,
-        validator=validator.is_not_zero,
-        fixer=RETURN_DEFAULT,
-    ),
-    'rotation': DXFAttr(50, default=0, optional=True),
-    'column_count': DXFAttr(
-        70, default=1, optional=True,
-        validator=validator.is_greater_zero,
-        fixer=RETURN_DEFAULT,
-    ),
-    'row_count': DXFAttr(
-        71, default=1, optional=True,
-        validator=validator.is_greater_zero,
-        fixer=RETURN_DEFAULT,
-    ),
-    'column_spacing': DXFAttr(44, default=0, optional=True),
-    'row_spacing': DXFAttr(45, default=0, optional=True),
-    'extrusion': DXFAttr(
-        210, xtype=XType.point3d, default=Z_AXIS, optional=True,
-        validator=validator.is_not_null_vector,
-        fixer=RETURN_DEFAULT,
-    ),
-})
+acdb_block_reference = DefSubclass(
+    "AcDbBlockReference",
+    {
+        "attribs_follow": DXFAttr(66, default=0, optional=True),
+        "name": DXFAttr(2, validator=validator.is_valid_block_name),
+        "insert": DXFAttr(10, xtype=XType.any_point),
+        # Elevation is a legacy feature from R11 and prior, do not use this
+        # attribute, store the entity elevation in the z-axis of the vertices.
+        # ezdxf does not export the elevation attribute!
+        "elevation": DXFAttr(38, default=0, optional=True),
+        "xscale": DXFAttr(
+            41,
+            default=1,
+            optional=True,
+            validator=validator.is_not_zero,
+            fixer=RETURN_DEFAULT,
+        ),
+        "yscale": DXFAttr(
+            42,
+            default=1,
+            optional=True,
+            validator=validator.is_not_zero,
+            fixer=RETURN_DEFAULT,
+        ),
+        "zscale": DXFAttr(
+            43,
+            default=1,
+            optional=True,
+            validator=validator.is_not_zero,
+            fixer=RETURN_DEFAULT,
+        ),
+        "rotation": DXFAttr(50, default=0, optional=True),
+        "column_count": DXFAttr(
+            70,
+            default=1,
+            optional=True,
+            validator=validator.is_greater_zero,
+            fixer=RETURN_DEFAULT,
+        ),
+        "row_count": DXFAttr(
+            71,
+            default=1,
+            optional=True,
+            validator=validator.is_greater_zero,
+            fixer=RETURN_DEFAULT,
+        ),
+        "column_spacing": DXFAttr(44, default=0, optional=True),
+        "row_spacing": DXFAttr(45, default=0, optional=True),
+        "extrusion": DXFAttr(
+            210,
+            xtype=XType.point3d,
+            default=Z_AXIS,
+            optional=True,
+            validator=validator.is_not_null_vector,
+            fixer=RETURN_DEFAULT,
+        ),
+    },
+)
+acdb_block_reference_group_codes = group_code_mapping(acdb_block_reference)
+merged_insert_group_codes = merge_group_code_mappings(
+    acdb_entity_group_codes, acdb_block_reference_group_codes  # type: ignore
+)
 
-NON_ORTHO_MSG = 'INSERT entity can not represent a non-orthogonal target ' \
-                'coordinate system.'
+NON_ORTHO_MSG = (
+    "INSERT entity can not represent a non-orthogonal target "
+    "coordinate system."
+)
 
 
 # Notes to SEQEND:
@@ -93,50 +164,63 @@ NON_ORTHO_MSG = 'INSERT entity can not represent a non-orthogonal target ' \
 # Nonetheless the Insert.add_attrib() method also creates a requires SEQEND if
 # necessary.
 
+
 @factory.register_entity
 class Insert(LinkedEntities):
-    """ DXF INSERT entity """
-    DXFTYPE = 'INSERT'
+    """DXF INSERT entity"""
+
+    DXFTYPE = "INSERT"
     DXFATTRIBS = DXFAttributes(base_class, acdb_entity, acdb_block_reference)
 
     @property
-    def attribs(self):
-        return self._sub_entities
+    def attribs(self) -> List[Attrib]:
+        return self._sub_entities  # type: ignore
 
     @property
     def attribs_follow(self) -> bool:
         return bool(len(self.attribs))
 
-    def load_dxf_attribs(self,
-                         processor: SubclassProcessor = None) -> 'DXFNamespace':
-        dxf = super().load_dxf_attribs(processor)
+    def load_dxf_attribs(
+        self, processor: SubclassProcessor = None
+    ) -> DXFNamespace:
+        """Loading interface. (internal API)"""
+        # bypass DXFGraphic, loading proxy graphic is skipped!
+        dxf = super(DXFGraphic, self).load_dxf_attribs(processor)
         if processor:
-            # Always use the 2nd subclass, could be AcDbBlockReference or
-            # AcDbMInsertBlock:
-            tags = processor.load_dxfattribs_into_namespace(
-                dxf, acdb_block_reference, 2)
-            if len(tags) and not processor.r12:
-                processor.log_unprocessed_tags(
-                    tags, subclass=acdb_block_reference.name)
+            processor.simple_dxfattribs_loader(dxf, merged_insert_group_codes)
+            if processor.r12:
+                # Transform elevation attribute from R11 to z-axis values:
+                elevation_to_z_axis(dxf, ("insert",))
         return dxf
 
-    def export_entity(self, tagwriter: 'TagWriter') -> None:
-        """ Export entity specific data as DXF tags. """
+    def export_entity(self, tagwriter: TagWriter) -> None:
+        """Export entity specific data as DXF tags."""
         super().export_entity(tagwriter)
         if tagwriter.dxfversion > DXF12:
             if (self.dxf.column_count > 1) or (self.dxf.row_count > 1):
-                tagwriter.write_tag2(SUBCLASS_MARKER, 'AcDbMInsertBlock')
+                tagwriter.write_tag2(SUBCLASS_MARKER, "AcDbMInsertBlock")
             else:
-                tagwriter.write_tag2(SUBCLASS_MARKER, 'AcDbBlockReference')
+                tagwriter.write_tag2(SUBCLASS_MARKER, "AcDbBlockReference")
         if self.attribs_follow:
             tagwriter.write_tag2(66, 1)
-        self.dxf.export_dxf_attribs(tagwriter, [
-            'name', 'insert', 'xscale', 'yscale', 'zscale', 'rotation',
-            'column_count', 'row_count', 'column_spacing', 'row_spacing',
-            'extrusion',
-        ])
+        self.dxf.export_dxf_attribs(
+            tagwriter,
+            [
+                "name",
+                "insert",
+                "xscale",
+                "yscale",
+                "zscale",
+                "rotation",
+                "column_count",
+                "row_count",
+                "column_spacing",
+                "row_spacing",
+                "extrusion",
+            ],
+        )
 
-    def export_dxf(self, tagwriter: 'TagWriter'):
+    def export_dxf(self, tagwriter: TagWriter):
         super().export_dxf(tagwriter)
         # Do no export SEQEND if no ATTRIBS attached:
         if self.attribs_follow:
@@ -144,54 +228,53 @@ class Insert(LinkedEntities):
 
     @property
     def has_scaling(self) -> bool:
-        """ Returns ``True`` if any axis scaling is applied.
-
-        .. versionadded:: 0.12
-
-        """
-        if self.dxf.hasattr('xscale') and self.dxf.xscale != 1:
+        """Returns ``True`` if any axis scaling is applied."""
+        if self.dxf.hasattr("xscale") and self.dxf.xscale != 1:
             return True
-        if self.dxf.hasattr('yscale') and self.dxf.yscale != 1:
+        if self.dxf.hasattr("yscale") and self.dxf.yscale != 1:
             return True
-        if self.dxf.hasattr('zscale') and self.dxf.zscale != 1:
+        if self.dxf.hasattr("zscale") and self.dxf.zscale != 1:
             return True
         return False
 
     @property
     def has_uniform_scaling(self) -> bool:
-        """ Returns ``True`` if scaling is uniform in x-, y- and z-axis ignoring
+        """Returns ``True`` if scaling is uniform in x-, y- and z-axis ignoring
         reflections e.g. (1, 1, -1) is uniform scaling.
 
         """
-        return abs(self.dxf.xscale) == abs(self.dxf.yscale) == abs(
-            self.dxf.zscale)
+        return (
+            abs(self.dxf.xscale) == abs(self.dxf.yscale) == abs(self.dxf.zscale)
+        )
 
     def set_scale(self, factor: float):
-        """ Set uniform scaling. """
+        """Set uniform scaling."""
         if factor == 0:
-            raise ValueError('Invalid scaling factor.')
+            raise ValueError("Invalid scaling factor.")
         self.dxf.xscale = factor
         self.dxf.yscale = factor
         self.dxf.zscale = factor
         return self
 
+    def block(self) -> Optional[BlockLayout]:
+        """Returns associated :class:`~ezdxf.layouts.BlockLayout`."""
+        if self.doc:
+            return self.doc.blocks.get(self.dxf.name)
+        return None
+
     def is_xref(self) -> bool:
-        """ Return ``True`` if XREF or XREF_OVERLAY. """
-        assert self.doc is not None, 'Requires a document object'
-        block_layout = self.doc.blocks.get(self.dxf.name)
-        if block_layout is not None and block_layout.block.dxf.flags & 12:  # XREF(4) & XREF_OVERLAY(8)
-            return True
+        """Return ``True`` if XREF or XREF_OVERLAY."""
+        block = self.block()
+        if block is not None:
+            return block.block_record.is_xref
         return False
 
-    def block(self) -> Optional['BlockLayout']:
-        """  Returns associated :class:`~ezdxf.layouts.BlockLayout`. """
-        if self.doc is None:
-            return None
-        return self.doc.blocks.get(self.dxf.name)
-
-    def place(self, insert: 'Vertex' = None,
-              scale: Tuple[float, float, float] = None,
-              rotation: float = None) -> 'Insert':
+    def place(
+        self,
+        insert: UVec = None,
+        scale: Tuple[float, float, float] = None,
+        rotation: float = None,
+    ) -> Insert:
         """
         Set block reference placing location `insert`, scaling and rotation
         attributes. Parameters which are ``None`` will not be altered.
@@ -217,9 +300,12 @@ class Insert(LinkedEntities):
             self.dxf.rotation = rotation
         return self
 
-    def grid(self, size: Tuple[int, int] = (1, 1),
-             spacing: Tuple[float, float] = (1, 1)) -> 'Insert':
-        """ Place block reference in a grid layout, grid `size` defines the
+    def grid(
+        self,
+        size: Tuple[int, int] = (1, 1),
+        spacing: Tuple[float, float] = (1, 1),
+    ) -> Insert:
+        """Place block reference in a grid layout, grid `size` defines the
         row- and column count, `spacing` defines the distance between two block
         references.
 
@@ -247,9 +333,10 @@ class Insert(LinkedEntities):
         self.dxf.column_spacing = col_spacing
         return self
 
-    def get_attrib(self, tag: str, search_const: bool = False) -> Optional[
-        Union['Attrib', 'AttDef']]:
-        """ Get attached :class:`Attrib` entity with :code:`dxf.tag == tag`,
+    def get_attrib(
+        self, tag: str, search_const: bool = False
+    ) -> Optional[Union[Attrib, AttDef]]:
+        """Get attached :class:`Attrib` entity with :code:`dxf.tag == tag`,
         returns ``None`` if not found. Some applications may not attach constant
         ATTRIB entities, set `search_const` to ``True``, to get at least the
         associated :class:`AttDef` entity.
@@ -269,9 +356,10 @@ class Insert(LinkedEntities):
                     return attdef
         return None
 
-    def get_attrib_text(self, tag: str, default: str = None,
-                        search_const: bool = False) -> str:
-        """ Get content text of attached :class:`Attrib` entity with
+    def get_attrib_text(
+        self, tag: str, default: str = "", search_const: bool = False
+    ) -> str:
+        """Get content text of attached :class:`Attrib` entity with
         :code:`dxf.tag == tag`, returns `default` if not found.
         Some applications may not attach constant ATTRIB entities, set
         `search_const` to ``True``, to get content text of the
@@ -289,7 +377,7 @@ class Insert(LinkedEntities):
         return attrib.dxf.text
 
     def has_attrib(self, tag: str, search_const: bool = False) -> bool:
-        """ Returns ``True`` if ATTRIB `tag` exist, for `search_const` doc see
+        """Returns ``True`` if ATTRIB `tag` exist, for `search_const` doc see
         :meth:`get_attrib`.
 
         Args:
@@ -299,15 +387,20 @@ class Insert(LinkedEntities):
         """
         return self.get_attrib(tag, search_const) is not None
 
-    def add_attrib(self, tag: str, text: str, insert: 'Vertex' = (0, 0),
-                   dxfattribs: dict = None) -> 'Attrib':
-        """ Attach an :class:`Attrib` entity to the block reference.
+    def add_attrib(
+        self,
+        tag: str,
+        text: str,
+        insert: UVec = (0, 0),
+        dxfattribs=None,
+    ) -> "Attrib":
+        """Attach an :class:`Attrib` entity to the block reference.
 
         Example for appending an attribute to an INSERT entity with none
         standard alignment::
 
-            e.add_attrib('EXAMPLETAG', 'example text').set_pos(
-                (3, 7), align='MIDDLE_CENTER'
+            e.add_attrib('EXAMPLETAG', 'example text').set_placement(
+                (3, 7), align=TextEntityAlignment.MIDDLE_CENTER
             )
 
         Args:
@@ -317,12 +410,11 @@ class Insert(LinkedEntities):
             dxfattribs: additional DXF attributes for the ATTRIB entity
 
         """
-        dxfattribs = dxfattribs or {}
-        dxfattribs['tag'] = tag
-        dxfattribs['text'] = text
-        dxfattribs['insert'] = insert
-        attrib = cast('Attrib',
-                      self._new_compound_entity('ATTRIB', dxfattribs))
+        dxfattribs = dict(dxfattribs or {})
+        dxfattribs["tag"] = tag
+        dxfattribs["text"] = text
+        dxfattribs["insert"] = insert
+        attrib = cast("Attrib", self._new_compound_entity("ATTRIB", dxfattribs))
         self.attribs.append(attrib)
 
         # This case is only possible if INSERT is read from file without
@@ -332,7 +424,7 @@ class Insert(LinkedEntities):
         return attrib
 
     def delete_attrib(self, tag: str, ignore=False) -> None:
-        """ Delete an attached :class:`Attrib` entity from INSERT. If `ignore`
+        """Delete an attached :class:`Attrib` entity from INSERT. If `ignore`
         is ``False``, an :class:`DXFKeyError` exception is raised, if
         ATTRIB `tag` does not exist.
 
@@ -354,8 +446,7 @@ class Insert(LinkedEntities):
             raise DXFKeyError(tag)
 
     def delete_all_attribs(self) -> None:
-        """ Delete all :class:`Attrib` entities attached to the INSERT entity.
-        """
+        """Delete all :class:`Attrib` entities attached to the INSERT entity."""
         if not self.is_alive:
             return
 
@@ -363,50 +454,46 @@ class Insert(LinkedEntities):
             attrib.destroy()
         self._sub_entities = []
 
-    def transform(self, m: 'Matrix44') -> 'Insert':
-        """ Transform INSERT entity by transformation matrix `m` inplace.
+    def transform(self, m: Matrix44) -> Insert:
+        """Transform INSERT entity by transformation matrix `m` inplace.
 
         Unlike the transformation matrix `m`, the INSERT entity can not
-        represent a non orthogonal target coordinate system, for this case an
+        represent a non-orthogonal target coordinate system, for this case an
         :class:`InsertTransformationError` will be raised.
 
-        .. versionadded:: 0.13
-
         """
 
         dxf = self.dxf
-        m1 = self.matrix44()
+        ocs = self.ocs()
 
-        # Transform scaled source axis into target coordinate system
-        ux, uy, uz = m.transform_directions((m1.ux, m1.uy, m1.uz))
+        # Transform source OCS axis into the target coordinate system:
+        ux, uy, uz = m.transform_directions((ocs.ux, ocs.uy, ocs.uz))
 
-        # Get new scaling factors, all are positive:
-        # z-axis is the real new z-axis, no reflection required
-        # x-axis is the real new x-axis, no reflection required
-        # y-axis - reflection is detected below
-        z_scale = uz.magnitude
-        x_scale = ux.magnitude
-        y_scale = uy.magnitude
+        # Calculate new axis scaling factors:
+        x_scale = ux.magnitude * dxf.xscale
+        y_scale = uy.magnitude * dxf.yscale
+        z_scale = uz.magnitude * dxf.zscale
 
-        # check for orthogonal x-, y- and z-axis
         ux = ux.normalize()
         uy = uy.normalize()
         uz = uz.normalize()
-        if not (math.isclose(ux.dot(uz), 0.0, abs_tol=1e-9) and
-                math.isclose(ux.dot(uy), 0.0, abs_tol=1e-9) and
-                math.isclose(uz.dot(uy), 0.0, abs_tol=1e-9)):
+        # check for orthogonal x-, y- and z-axis
+        if (
+            abs(ux.dot(uz)) > ABS_TOL
+            or abs(ux.dot(uy)) > ABS_TOL
+            or abs(uz.dot(uy)) > ABS_TOL
+        ):
             raise InsertTransformationError(NON_ORTHO_MSG)
 
-        # expected y-axis for an orthogonal right handed coordinate system
+        # expected y-axis for an orthogonal right handed coordinate system:
         expected_uy = uz.cross(ux)
-        if expected_uy.isclose(-uy, abs_tol=1e-9):
-            # transformed y-axis points into opposite direction of the expected
-            # y-axis:
+        if not expected_uy.isclose(uy, abs_tol=ABS_TOL):
+            # new y-axis points into opposite direction:
             y_scale = -y_scale
 
-        ocs = OCSTransform.from_ocs(OCS(dxf.extrusion), OCS(uz), m)
-        dxf.insert = ocs.transform_vertex(dxf.insert)
-        dxf.rotation = ocs.transform_deg_angle(dxf.rotation)
+        ocs_transform = OCSTransform.from_ocs(OCS(dxf.extrusion), OCS(uz), m)
+        dxf.insert = ocs_transform.transform_vertex(dxf.insert)
+        dxf.rotation = ocs_transform.transform_deg_angle(dxf.rotation)
 
         dxf.extrusion = uz
         dxf.xscale = x_scale
@@ -415,27 +502,25 @@ class Insert(LinkedEntities):
 
         for attrib in self.attribs:
             attrib.transform(m)
+        self.post_transform(m)
         return self
 
-    def translate(self, dx: float, dy: float, dz: float) -> 'Insert':
-        """ Optimized INSERT translation about `dx` in x-axis, `dy` in y-axis
+    def translate(self, dx: float, dy: float, dz: float) -> Insert:
+        """Optimized INSERT translation about `dx` in x-axis, `dy` in y-axis
         and `dz` in z-axis.
 
-        .. versionadded:: 0.13
-
         """
         ocs = self.ocs()
         self.dxf.insert = ocs.from_wcs(
-            Vector(dx, dy, dz) + ocs.to_wcs(self.dxf.insert))
+            Vec3(dx, dy, dz) + ocs.to_wcs(self.dxf.insert)
+        )
         for attrib in self.attribs:
             attrib.translate(dx, dy, dz)
         return self
 
     def matrix44(self) -> Matrix44:
-        """ Returns a transformation :class:`Matrix44` object to transform block
-        entities into WCS.
-
-        .. versionadded:: 0.13
+        """Returns a transformation matrix of type :class:`Matrix44` to
+        transform the block entities into :ref:`WCS`.
 
         """
         dxf = self.dxf
@@ -445,27 +530,28 @@ class Insert(LinkedEntities):
 
         ocs = self.ocs()
         extrusion = ocs.uz
-        ux = Vector(ocs.to_wcs(X_AXIS))
-        uy = Vector(ocs.to_wcs(Y_AXIS))
+        ux = Vec3(ocs.to_wcs(X_AXIS))
+        uy = Vec3(ocs.to_wcs(Y_AXIS))
         m = Matrix44.ucs(ux=ux * sx, uy=uy * sy, uz=extrusion * sz)
+        # same as Matrix44.ucs(ux, uy, extrusion) * Matrix44.scale(sx, sy, sz)
 
         angle = math.radians(dxf.rotation)
-        if angle != 0.0:
-            m = Matrix44.chain(m, Matrix44.axis_rotate(extrusion, angle))
+        if angle:
+            m *= Matrix44.axis_rotate(extrusion, angle)
 
-        insert = ocs.to_wcs(dxf.get('insert', Vector()))
+        insert = ocs.to_wcs(dxf.get("insert", NULLVEC))
 
         block_layout = self.block()
         if block_layout is not None:
             # transform block base point into WCS without translation
-            insert -= m.transform_direction(block_layout.block.dxf.base_point)
+            insert -= m.transform_direction(block_layout.block.dxf.base_point)  # type: ignore
 
         # set translation
-        m.set_row(3, insert.xyz)
+        m.set_row(3, insert.xyz)  # type: ignore
         return m
 
     def ucs(self):
-        """ Returns the block reference coordinate system as
+        """Returns the block reference coordinate system as
         :class:`ezdxf.math.UCS` object.
         """
         m = self.matrix44()
@@ -474,20 +560,20 @@ class Insert(LinkedEntities):
         return ucs
 
     def reset_transformation(self) -> None:
-        """ Reset block reference parameters `location`, `rotation` and
+        """Reset block reference parameters `location`, `rotation` and
         `extrusion` vector.
 
         """
         self.dxf.insert = NULLVEC
-        self.dxf.discard('rotation')
-        self.dxf.discard('extrusion')
+        self.dxf.discard("rotation")
+        self.dxf.discard("extrusion")
 
-    def explode(self, target_layout: 'BaseLayout' = None,
-                non_uniform_scaling=None) -> 'EntityQuery':
-        """
-        Explode block reference entities into target layout, if target layout is
-        ``None``, the target layout is the layout of the block reference.
-        This method destroys the source block reference entity.
+    def explode(
+        self, target_layout: BaseLayout = None, *, redraw_order=False
+    ) -> EntityQuery:
+        """Explode block reference entities into target layout, if target
+        layout is ``None``, the target layout is the layout of the block
+        reference. This method destroys the source block reference entity.
 
         Transforms the block entities into the required :ref:`WCS` location by
         applying the block reference attributes `insert`, `extrusion`,
@@ -506,31 +592,36 @@ class Insert(LinkedEntities):
 
         Args:
             target_layout: target layout for exploded entities, ``None`` for
-            same layout as source entity.
-
-        .. versionchanged:: 0.13
-            deprecated `non_uniform_scaling` argument
+                same layout as source entity.
+            redraw_order: create entities in ascending redraw order if ``True``
 
         """
         if target_layout is None:
             target_layout = self.get_layout()
             if target_layout is None:
                 raise DXFStructureError(
-                    'INSERT without layout assigment, specify target layout.'
+                    "INSERT without layout assignment, specify target layout."
                 )
-        if non_uniform_scaling is not None:
-            warnings.warn(
-                'Insert.explode() argument `non_uniform_scaling` is deprecated'
-                ' (removed in v0.15).',
-                DeprecationWarning
-            )
-        return explode_block_reference(self, target_layout=target_layout)
+        return explode_block_reference(
+            self, target_layout=target_layout, redraw_order=redraw_order
+        )
 
-    def virtual_entities(self,
-                         non_uniform_scaling=None,
-                         skipped_entity_callback: Optional[
-                             Callable[[DXFGraphic, str], None]] = None
-                         ) -> Iterable[DXFGraphic]:
+    def __virtual_entities__(self) -> Iterable[DXFGraphic]:
+        """Implements the SupportsVirtualEntities protocol.
+
+        This protocol is for consistent internal usage and does not replace
+        the method :meth:`virtual_entities`! Ignores the redraw order!
+        """
+        return self.virtual_entities()
+
+    def virtual_entities(
+        self,
+        *,
+        skipped_entity_callback: Optional[
+            Callable[[DXFGraphic, str], None]
+        ] = None,
+        redraw_order=False,
+    ) -> Iterable[DXFGraphic]:
         """
         Yields "virtual" entities of a block reference. This method is meant to
         examine the block reference entities at the "exploded" location without
@@ -564,37 +655,30 @@ class Insert(LinkedEntities):
         Args:
             skipped_entity_callback: called whenever the transformation of an
                 entity is not supported and so was skipped
-
-        .. versionchanged:: 0.13
-            deprecated `non_uniform_scaling` argument
+            redraw_order: yield entities in ascending redraw order if ``True``
 
         """
-        if non_uniform_scaling is not None:
-            warnings.warn(
-                'Insert.virtual_entities() argument `non_uniform_scaling` is'
-                ' deprecated (removed in v0.15).',
-                DeprecationWarning
-            )
-        return virtual_block_reference_entities(
-            self, skipped_entity_callback=skipped_entity_callback)
+        for e in virtual_block_reference_entities(
+            self,
+            skipped_entity_callback=skipped_entity_callback,
+            redraw_order=redraw_order,
+        ):
+            e.set_source_block_reference(self)
+            yield e
 
     @property
-    def mcount(self):
-        """ Returns the multi-insert count, MINSERT (multi-insert) processing
+    def mcount(self) -> int:
+        """Returns the multi-insert count, MINSERT (multi-insert) processing
         is required if :attr:`mcount` > 1.
 
-        .. versionadded:: 0.14
-
         """
         return (self.dxf.row_count if self.dxf.row_spacing else 1) * (
-            self.dxf.column_count if self.dxf.column_spacing else 1)
+            self.dxf.column_count if self.dxf.column_spacing else 1
+        )
 
-    def multi_insert(self) -> Iterable['Insert']:
-        """ Yields a virtual INSERT entity for each grid element of a MINSERT
+    def multi_insert(self) -> Iterable[Insert]:
+        """Yields a virtual INSERT entity for each grid element of a MINSERT
         entity (multi-insert).
-
-        .. versionadded:: 0.14
-
         """
 
         def transform_attached_attrib_entities(insert, offset):
@@ -604,10 +688,10 @@ class Insert(LinkedEntities):
         def adjust_dxf_attribs(insert, offset):
             dxf = insert.dxf
             dxf.insert += offset
-            dxf.discard('row_count')
-            dxf.discard('column_count')
-            dxf.discard('row_spacing')
-            dxf.discard('column_spacing')
+            dxf.discard("row_count")
+            dxf.discard("column_count")
+            dxf.discard("row_spacing")
+            dxf.discard("column_spacing")
 
         done = set()
         row_spacing = self.dxf.row_spacing
@@ -616,7 +700,7 @@ class Insert(LinkedEntities):
         for row in range(self.dxf.row_count):
             for col in range(self.dxf.column_count):
                 # All transformations in OCS:
-                offset = Vector(col * col_spacing, row * row_spacing)
+                offset = Vec3(col * col_spacing, row * row_spacing)
                 # If any spacing is 0, yield only unique locations:
                 if offset not in done:
                     done.add(offset)
@@ -628,7 +712,7 @@ class Insert(LinkedEntities):
                     transform_attached_attrib_entities(insert, offset)
                     yield insert
 
-    def add_auto_attribs(self, values: Dict[str, str]) -> 'Insert':
+    def add_auto_attribs(self, values: Dict[str, str]) -> Insert:
         """
         Attach for each :class:`~ezdxf.entities.Attdef` entity, defined in the
         block definition, automatically an :class:`Attrib` entity to the block
@@ -649,33 +733,48 @@ class Insert(LinkedEntities):
 
         """
 
-        def unpack(dxfattribs) -> Tuple[str, str, 'Vertex']:
-            tag = dxfattribs.pop('tag')
+        def unpack(dxfattribs) -> Tuple[str, str, UVec]:
+            tag = dxfattribs.pop("tag")
             text = values.get(tag, "")
-            location = dxfattribs.pop('insert')
+            location = dxfattribs.pop("insert")
             return tag, text, location
 
         def autofill() -> None:
-            for attdef in blockdef.attdefs():
-                dxfattribs = attdef.dxfattribs(drop={'prompt', 'handle'})
+            for attdef in block_layout.attdefs():  # type: ignore
+                dxfattribs = attdef.dxfattribs(drop={"prompt", "handle"})
                 tag, text, location = unpack(dxfattribs)
                 attrib = self.add_attrib(tag, text, location, dxfattribs)
                 attrib.transform(m)
 
-        m = self.matrix44()
-        blockdef = self.block()
-        autofill()
+        block_layout = self.block()
+        if block_layout is not None:
+            m = self.matrix44()
+            autofill()
         return self
 
-    def audit(self, auditor: 'Auditor') -> None:
-        """ Validity check. """
+    def audit(self, auditor: Auditor) -> None:
+        """Validity check."""
         super().audit(auditor)
         doc = auditor.doc
         if doc and doc.blocks:
-            if self.dxf.name not in doc.blocks:
+            name = self.dxf.name
+            if name is None:
                 auditor.fixed_error(
-                    code=AuditError.UNDEFINED_BLOCK,
-                    message=f'Deleted entity {str(self)} without required BLOCK'
-                            f' definition.',
+                    code=AuditError.UNDEFINED_BLOCK_NAME,
+                    message=f"Deleted entity {str(self)} without a BLOCK name",
                 )
                 auditor.trash(self)
+            elif name not in doc.blocks:
+                auditor.fixed_error(
+                    code=AuditError.UNDEFINED_BLOCK,
+                    message=f"Deleted entity {str(self)} without required BLOCK"
+                    f" definition.",
+                )
+                auditor.trash(self)
+
+    def __referenced_blocks__(self) -> Iterable[str]:
+        """Support for the "ReferencedBlocks" protocol."""
+        block = self.block()
+        if block is not None:
+            return (block.block_record_handle,)
+        return tuple()
