@@ -1,22 +1,34 @@
-# Created: 2019-02-18
-# Copyright (c) 2019-2020, Manfred Moitzi
+# Copyright (c) 2019-2022, Manfred Moitzi
 # License: MIT License
+from __future__ import annotations
 from typing import (
-    TYPE_CHECKING, Iterable, Any, Union, List, Tuple, cast, Optional, Dict,
+    TYPE_CHECKING,
+    Any,
+    Union,
+    List,
+    Tuple,
+    cast,
+    Optional,
 )
+from ezdxf.math import Vec2, UVec
 from ezdxf.entitydb import EntitySpace
 from ezdxf.lldxf import const
 from .base import BaseLayout
 
 if TYPE_CHECKING:
     from ezdxf.eztypes import (
-        GeoData, SortEntsTable, Vertex, Viewport, Drawing, DXFLayout,
-        DXFGraphic, BlockLayout,
+        GeoData,
+        Viewport,
+        Drawing,
+        DXFLayout,
+        DXFGraphic,
+        BlockLayout,
     )
 
 
-def get_block_entity_space(doc: 'Drawing',
-                           block_record_handle: str) -> 'EntitySpace':
+def get_block_entity_space(
+    doc: "Drawing", block_record_handle: str
+) -> "EntitySpace":
     block_record = doc.entitydb[block_record_handle]
     return block_record.entity_space
 
@@ -48,6 +60,7 @@ class Layout(BaseLayout):
     of the :class:`Layout` object are the same object.
 
     """
+
     # plot_layout_flags of LAYOUT entity
     PLOT_VIEWPORT_BORDERS = 1
     SHOW_PLOT_STYLES = 2
@@ -64,7 +77,7 @@ class Layout(BaseLayout):
     INITIALIZING = 8192
     PREV_PLOT_INIT = 16384
 
-    def __init__(self, layout: 'DXFLayout', doc: 'Drawing'):
+    def __init__(self, layout: "DXFLayout", doc: "Drawing"):
         self.dxf_layout = layout
         block_record = doc.entitydb[layout.dxf.block_record_handle]
         # link maybe broken
@@ -72,9 +85,10 @@ class Layout(BaseLayout):
         super().__init__(block_record)
 
     @classmethod
-    def new(cls, name: str, block_name: str, doc: 'Drawing',
-            dxfattribs: dict = None) -> 'Layout':
-        """ Returns the required structures for a new layout:
+    def new(
+        cls, name: str, block_name: str, doc: "Drawing", dxfattribs: dict = None
+    ) -> "Layout":
+        """Returns the required structures for a new layout:
 
             - a :class:`BlockLayout` with BLOCK_RECORD, BLOCK and ENDBLK entities
             - LAYOUT entity in the objects section
@@ -90,28 +104,30 @@ class Layout(BaseLayout):
         """
         block_layout = doc.blocks.new(block_name)  # type: BlockLayout
         dxfattribs = dxfattribs or {}
-        dxfattribs.update({
-            'name': name,
-            'block_record_handle': block_layout.block_record_handle
-        })
-        dxf_layout = doc.objects.new_entity('LAYOUT', dxfattribs=dxfattribs)
-        return cls(dxf_layout, doc)
+        dxfattribs.update(
+            {
+                "name": name,
+                "block_record_handle": block_layout.block_record_handle,
+            }
+        )
+        dxf_layout = doc.objects.new_entity("LAYOUT", dxfattribs=dxfattribs)
+        return cls(dxf_layout, doc)  # type: ignore
 
     @classmethod
-    def load(cls, layout: 'DXFLayout', doc: 'Drawing'):
-        """ Loading interface. (internal API) """
-        layout = cls(layout, doc)
-        layout._repair_owner_tags()
-        return layout
+    def load(cls, layout: "DXFLayout", doc: "Drawing"):
+        """Loading interface. (internal API)"""
+        _layout = cls(layout, doc)
+        _layout._repair_owner_tags()
+        return _layout
 
     @property
     def name(self) -> str:
-        """ Layout name as shown in tabs of :term:`CAD` applications. """
+        """Layout name as shown in tabs of :term:`CAD` applications."""
         return self.dxf_layout.dxf.name
 
     @property  # dynamic DXF attribute dispatching, e.g. DXFLayout.dxf.layout_flags
     def dxf(self) -> Any:
-        """ Returns the DXF name space attribute of the associated
+        """Returns the DXF name space attribute of the associated
         :class:`~ezdxf.entities.DXFLayout` object.
 
         This enables direct access to the underlying LAYOUT entity,
@@ -122,11 +138,11 @@ class Layout(BaseLayout):
 
     @property
     def block_record_name(self) -> str:
-        """ Returns the name of the associated BLOCK_RECORD as string. """
+        """Returns the name of the associated BLOCK_RECORD as string."""
         return self.block_record.dxf.name
 
     def _repair_owner_tags(self) -> None:
-        """ Set `owner` and `paperspace` attributes of entities hosted by this
+        """Set `owner` and `paperspace` attributes of entities hosted by this
         layout to correct values.
         """
         layout_key = self.layout_key
@@ -137,19 +153,19 @@ class Layout(BaseLayout):
             if entity.dxf.paperspace != paperspace:
                 entity.dxf.paperspace = paperspace
 
-    def __contains__(self, entity: Union['DXFGraphic', str]) -> bool:
-        """ Returns ``True`` if `entity` is stored in this layout.
+    def __contains__(self, entity: Union["DXFGraphic", str]) -> bool:
+        """Returns ``True`` if `entity` is stored in this layout.
 
         Args:
              entity: :class:`DXFGraphic` object or handle as hex string
 
         """
         if isinstance(entity, str):  # entity is a handle string
-            entity = self.entitydb[entity]
-        return entity.dxf.owner == self.layout_key
+            entity = self.entitydb[entity]  # type: ignore
+        return entity.dxf.owner == self.layout_key  # type: ignore
 
     def destroy(self) -> None:
-        """ Delete all entities and the layout itself from entity database and
+        """Delete all entities and the layout itself from entity database and
         all linked structures.
 
         (internal API)
@@ -159,85 +175,51 @@ class Layout(BaseLayout):
         # super() deletes block_record and associated entity space
         super().destroy()
 
-    def get_sortents_table(self, create: bool = True) -> 'SortEntsTable':
-        """ Get/Create the SORTENTSTABLE object associated to the layout.
+    @property
+    def plot_layout_flags(self) -> int:
+        return self.dxf_layout.dxf.plot_layout_flags
+
+    def reset_extents(
+        self, extmin=(+1e20, +1e20, +1e20), extmax=(-1e20, -1e20, -1e20)
+    ) -> None:
+        """Reset `extents`_ to given values or the AutoCAD default values.
+
+        "Drawing extents are the bounds of the area occupied by objects."
+        (Quote Autodesk Knowledge Network)
 
         Args:
-            create: new table if table do not exist and `create` is ``True``
-
-        Raises:
-            DXFValueError: if table not exist and `create` is ``False``
-
-        (internal API)
-        """
-        xdict = self.get_extension_dict()
-        try:
-            sortents_table = xdict['ACAD_SORTENTS']
-        except const.DXFKeyError:
-            if create:
-                sortents_table = self.doc.objects.new_entity(
-                    'SORTENTSTABLE',
-                    dxfattribs={
-                        'owner': xdict.dxf.handle,
-                        'block_record_handle': self.layout_key
-                    },
-                )
-                xdict['ACAD_SORTENTS'] = sortents_table.dxf.handle
-            else:
-                raise const.DXFValueError(
-                    'Extension dictionary entry ACAD_SORTENTS does not exist.')
-        return sortents_table
-
-    def set_redraw_order(self,
-                         handles: Union[Dict, Iterable[Tuple[str, str]]]
-                         ) -> None:
-        """ If the header variable $SORTENTS `Regen` flag (bit-code value 16)
-        is set, AutoCAD regenerates entities in ascending handles order.
-
-        To change redraw order associate a different sort handle to entities,
-        this redefines the order in which the entities are regenerated.
-        `handles` can be a dict of entity_handle and sort_handle as (k, v)
-        pairs, or an iterable of (entity_handle, sort_handle) tuples.
-
-        The sort_handle doesn't have to be unique, some or all entities can
-        share the same sort handle and a sort handle can be an existing handle.
-
-        The "0" handle can be used, but this sort_handle will be drawn as
-        latest (on top of all other entities) and not as first as expected.
-
-        Args:
-            handles: iterable or dict of handle associations; an iterable
-                of 2-tuples (entity_handle, sort_handle) or a dict (k, v)
-                association as (entity_handle, sort_handle)
+             extmin: minimum extents or (+1e20, +1e20, +1e20) as default value
+             extmax: maximum extents or (-1e20, -1e20, -1e20) as default value
 
         """
-        sortents = self.get_sortents_table()
-        if isinstance(handles, dict):
-            handles = handles.items()
-        sortents.set_handles(handles)
-
-    def get_redraw_order(self) -> Iterable[Tuple[str, str]]:
-        """ Returns iterable for all existing table entries as (entity_handle,
-        sort_handle) pairs, see also :meth:`~Layout.set_redraw_order`.
-
-        """
-        empty = []
-        if self.block_record.has_extension_dict:
-            xdict = self.get_extension_dict()
-        else:
-            return empty
-
-        try:
-            sortents_table = xdict['ACAD_SORTENTS']
-        except const.DXFKeyError:
-            return empty
-        return iter(sortents_table)
-
-    def reset_extends(self) -> None:
-        """ Reset extends. """
         dxf = self.dxf_layout.dxf
-        dxf.extmin = (+1e20, +1e20, +1e20)  # AutoCAD default
-        dxf.extmax = (-1e20, -1e20, -1e20)  # AutoCAD default
+        dxf.extmin = extmin
+        dxf.extmax = extmax
+
+    def reset_limits(self, limmin=None, limmax=None) -> None:
+        """Reset `limits`_ to given values or the AutoCAD default values.
+
+        "Sets an invisible rectangular boundary in the drawing area that can
+        limit the grid display and limit clicking or entering point locations."
+        (Quote Autodesk Knowledge Network)
+
+        The :class:`Paperspace` class has an additional method
+        :meth:`~Paperspace.reset_paper_limits` to deduce the default limits from
+        the paper size settings.
+
+        Args:
+             extmin: minimum extents or (0, 0) as default
+             extmax: maximum extents or (paper width, paper height) as default value
+
+        """
+        dxf = self.dxf_layout.dxf
+        if limmin is None:
+            limmin = (0, 0)
+        if limmax is None:
+            limmax = (dxf.paper_width, dxf.paper_height)
+
+        dxf.limmin = limmin
+        dxf.limmax = limmax
 
     def set_plot_type(self, value: int = 5) -> None:
         """
@@ -260,11 +242,12 @@ class Layout(BaseLayout):
         if 0 <= int(value) <= 5:
             self.dxf.plot_type = value
         else:
-            raise const.DXFValueError('Plot type value out of range (0-5).')
+            raise const.DXFValueError("Plot type value out of range (0-5).")
 
-    def set_plot_style(self, name: str = 'ezdxf.ctb',
-                       show: bool = False) -> None:
-        """ Set plot style file of type `.ctb`.
+    def set_plot_style(
+        self, name: str = "ezdxf.ctb", show: bool = False
+    ) -> None:
+        """Set plot style file of type `.ctb`.
 
         Args:
             name: plot style filename
@@ -275,10 +258,15 @@ class Layout(BaseLayout):
         self.use_plot_styles(True)
         self.show_plot_styles(show)
 
-    def set_plot_window(self,
-                        lower_left: Tuple[float, float] = (0, 0),
-                        upper_right: Tuple[float, float] = (0, 0)) -> None:
-        """ Set plot window size in (scaled) paper space units.
+    def get_plot_style_filename(self) -> str:
+        return self.dxf_layout.dxf.current_style_sheet
+
+    def set_plot_window(
+        self,
+        lower_left: Tuple[float, float] = (0, 0),
+        upper_right: Tuple[float, float] = (0, 0),
+    ) -> None:
+        """Set plot window size in (scaled) paper space units.
 
         Args:
             lower_left: lower left corner as 2D point
@@ -320,7 +308,7 @@ class Layout(BaseLayout):
         self.set_plot_flags(self.PRINT_LINEWEIGHTS, state)
 
     def draw_viewports_first(self, state: bool = True) -> None:
-        self.set_plot_flags(self.PRINT_LINEWEIGHTS, state)
+        self.set_plot_flags(self.DRAW_VIEWPORTS_FIRST, state)
 
     def model_type(self, state: bool = True) -> None:
         self.set_plot_flags(self.MODEL_TYPE, state)
@@ -338,12 +326,13 @@ class Layout(BaseLayout):
         self.set_plot_flags(self.PREV_PLOT_INIT, state)
 
     def set_plot_flags(self, flag, state: bool = True) -> None:
-        self.dxf_layout.set_flag_state(flag, state=state,
-                                       name='plot_layout_flags')
+        self.dxf_layout.set_flag_state(
+            flag, state=state, name="plot_layout_flags"
+        )
 
 
 class Modelspace(Layout):
-    """ :class:`Modelspace` - not deletable, all entities of this layout are
+    """:class:`Modelspace` - not deletable, all entities of this layout are
     stored in the ENTITIES section of the DXF file, the associated
     "*Model_Space" block is empty, block name is fixed as "*Model_Space",
     the name is fixed as ""Model".
@@ -352,11 +341,11 @@ class Modelspace(Layout):
 
     @property
     def name(self) -> str:
-        """ Name of modelspace is fixed as "Model". """
-        return 'Model'
+        """Name of modelspace is fixed as "Model"."""
+        return "Model"
 
-    def new_geodata(self, dxfattribs: dict = None) -> 'GeoData':
-        """ Creates a new :class:`GeoData` entity and replaces existing ones.
+    def new_geodata(self, dxfattribs: dict = None) -> "GeoData":
+        """Creates a new :class:`GeoData` entity and replaces existing ones.
         The GEODATA entity resides in the OBJECTS section and not in the
         modelspace, it is linked to the modelspace by an
         :class:`~ezdxf.entities.ExtensionDict` located in BLOCK_RECORD of the
@@ -373,7 +362,8 @@ class Modelspace(Layout):
         """
         if self.doc.dxfversion < const.DXF2010:
             raise const.DXFValueError(
-                'GEODATA entity requires DXF R2010 or later.')
+                "GEODATA entity requires DXF R2010 or later."
+            )
 
         if dxfattribs is None:
             dxfattribs = {}
@@ -382,11 +372,11 @@ class Modelspace(Layout):
             owner=xdict.dictionary.dxf.handle,
             dxfattribs=dxfattribs,
         )
-        xdict['ACAD_GEOGRAPHICDATA'] = geodata
+        xdict["ACAD_GEOGRAPHICDATA"] = geodata
         return geodata
 
-    def get_geodata(self) -> Optional['GeoData']:
-        """ Returns the :class:`~ezdxf.entities.GeoData` entity associated to
+    def get_geodata(self) -> Optional["GeoData"]:
+        """Returns the :class:`~ezdxf.entities.GeoData` entity associated to
         the modelspace or ``None``.
         """
         try:
@@ -394,13 +384,13 @@ class Modelspace(Layout):
         except AttributeError:
             return None
         try:
-            return xdict['ACAD_GEOGRAPHICDATA']
+            return xdict["ACAD_GEOGRAPHICDATA"]
         except const.DXFKeyError:
             return None
 
 
 class Paperspace(Layout):
-    """ There are two kind of paperspace layouts:
+    """There are two kind of paperspace layouts:
 
     1. Active Layout - all entities of this layout are stored in the ENTITIES
        section, the associated "*Paper_Space" block is empty, block name
@@ -419,58 +409,110 @@ class Paperspace(Layout):
     """
 
     def rename(self, name: str) -> None:
-        """ Rename layout to `name`, changes the name displayed in tabs by
-        CAD applications, not the internal BLOCK name.
+        """Rename layout to `name`, changes the name displayed in tabs by
+        CAD applications, not the internal BLOCK name. (internal API)
+
+        Use method :meth:`~ezdxf.layouts.Layouts.rename` of the
+        :meth:`~ezdxf.layouts.Layouts` class to rename paper space
+        layouts.
+
         """
         self.dxf_layout.dxf.name = name
 
-    def viewports(self) -> List['DXFGraphic']:
-        """ Get all VIEWPORT entities defined in the paperspace layout.
+    def viewports(self) -> List["Viewport"]:
+        """Get all VIEWPORT entities defined in this paperspace layout.
         Returns a list of :class:`~ezdxf.entities.Viewport` objects, sorted by
-        id, the first entity is always the paperspace view with an id of 1.
+        id, the first entity is always the main viewport with an id of 1.
 
         """
-        vports = [entity for entity in self if entity.dxftype() == 'VIEWPORT']
+        vports = [entity for entity in self if entity.dxftype() == "VIEWPORT"]
         vports.sort(key=lambda e: e.dxf.id)
-        return vports
+        return vports  # type: ignore
+
+    def main_viewport(self) -> Optional["Viewport"]:
+        """Returns the main viewport of this paper space layout, or ``None``
+        if no main viewport exist.
+
+        """
+        viewports = self.viewports()
+        if len(viewports):
+            return viewports[0]
+        else:
+            return None
 
     def renumber_viewports(self) -> None:
-        """ Reassign viewport ids. (internal API) """
+        """Reassign viewport ids. (internal API)"""
         for num, viewport in enumerate(self.viewports(), start=1):
             viewport.dxf.id = num
 
-    def add_viewport(self, center: 'Vertex',
-                     size: Tuple[float, float],
-                     view_center_point: 'Vertex',
-                     view_height: float,
-                     dxfattribs: dict = None) -> 'Viewport':
-        """ Add a new :class:`~ezdxf.entities.Viewport` entity. """
+    def add_viewport(
+        self,
+        center: UVec,
+        size: Tuple[float, float],
+        view_center_point: UVec,
+        view_height: float,
+        dxfattribs: dict = None,
+    ) -> "Viewport":
+        """Add a new :class:`~ezdxf.entities.Viewport` entity."""
         dxfattribs = dxfattribs or {}
         width, height = size
         attribs = {
-            'center': center,
-            'width': width,
-            'height': height,
-            'status': 1,  # by default highest priority (stack order)
-            'layer': 'VIEWPORTS',
+            "center": center,  # center in paperspace
+            "width": width,  # width in paperspace
+            "height": height,  # height in paperspace
+            "status": 1,  # has by default the highest priority (stack order)
+            "layer": "VIEWPORTS",
             # use separated layer to turn off for plotting
-            'view_center_point': view_center_point,
-            'view_height': view_height,
+            "view_center_point": view_center_point,  # in modelspace
+            "view_height": view_height,  # in modelspace
         }
         attribs.update(dxfattribs)
-        viewport = cast('Viewport', self.new_entity('VIEWPORT', attribs))
+        viewport = cast("Viewport", self.new_entity("VIEWPORT", attribs))
         viewport.dxf.id = viewport.get_next_viewport_id()
         return viewport
 
     def reset_viewports(self) -> None:
-        """ Delete all existing viewports, and add a new main viewport. """
+        """Delete all existing viewports, and create a new main viewport."""
         # remove existing viewports
         for viewport in self.viewports():
             self.delete_entity(viewport)
         self.add_new_main_viewport()
 
-    def add_new_main_viewport(self) -> None:
-        """ Add a new main viewport. (internal API) """
+    def reset_main_viewport(
+        self, center: UVec = None, size: UVec = None
+    ) -> "Viewport":
+        """Reset the main viewport of this paper space layout to the given
+        values, or reset them to the default values, deduced from the paper
+        settings. Creates a new main viewport if none exist.
+
+        Ezdxf does not create a main viewport by default, because CAD
+        applications don't require one.
+
+        Args:
+            center: center of the viewport in paper space units
+            size: viewport size as (width, height) tuple in paper space units
+
+        """
+        viewports = self.viewports()
+        if len(viewports) == 0 or viewports[0].dxf.id != 1:
+            viewport = self.add_new_main_viewport()
+        else:
+            viewport = viewports[0]
+        default_center, default_size = self.default_viewport_config()
+        if center is None:
+            center = default_center
+        if size is None:
+            size = default_size
+
+        viewport.dxf.center = center
+        width, height = size
+        viewport.dxf.width = width
+        viewport.dxf.height = height
+        return viewport
+
+    def default_viewport_config(
+        self,
+    ) -> Tuple[Tuple[float, float], Tuple[float, float]]:
         dxf = self.dxf_layout.dxf
         if dxf.plot_paper_units == 0:  # inches
             unit_factor = 25.4
@@ -491,10 +533,16 @@ class Paperspace(Layout):
         y_offset = paper_units(dxf.plot_origin_y_offset)
 
         # printing area
-        printable_width = paper_width - paper_units(
-            dxf.left_margin) - paper_units(dxf.right_margin)
-        printable_height = paper_height - paper_units(
-            dxf.bottom_margin) - paper_units(dxf.top_margin)
+        printable_width = (
+            paper_width
+            - paper_units(dxf.left_margin)
+            - paper_units(dxf.right_margin)
+        )
+        printable_height = (
+            paper_height
+            - paper_units(dxf.bottom_margin)
+            - paper_units(dxf.top_margin)
+        )
 
         # AutoCAD viewport (window) size
         vp_width = paper_width * 1.1
@@ -502,32 +550,42 @@ class Paperspace(Layout):
 
         # center of printing area
         center = (
-            printable_width / 2 - x_offset, printable_height / 2 - y_offset)
+            printable_width / 2 - x_offset,
+            printable_height / 2 - y_offset,
+        )
+        return center, (vp_width, vp_height)
 
+    def add_new_main_viewport(self) -> "Viewport":
+        """Add a new main viewport."""
+        center, size = self.default_viewport_config()
+        vp_height = size[1]
         # create 'main' viewport
         main_viewport = self.add_viewport(
             center=center,  # no influence to 'main' viewport?
-            size=(vp_width, vp_height),  # I don't get it, just use paper size!
+            size=size,  # I don't get it, just use paper size!
             view_center_point=center,  # same as center
             view_height=vp_height,  # view height in paper space units
         )
         main_viewport.dxf.id = 1  # set as main viewport
         main_viewport.dxf.flags = 557088  # AutoCAD default value
-        dxf.viewport_handle = main_viewport.dxf.handle
+        self.dxf_layout.dxf.viewport_handle = main_viewport.dxf.handle
+        return main_viewport
 
-    def page_setup(self, size: Tuple[float, float] = (297, 210),
-                   margins: Tuple[float, float, float, float] = (
-                           10, 15, 10, 15),
-                   units: str = 'mm',
-                   offset: Tuple[float, float] = (0, 0),
-                   rotation: int = 0,
-                   scale: int = 16,
-                   name: str = 'ezdxf',
-                   device: str = 'DWG to PDF.pc3') -> None:
-        """ Setup plot settings and paper size and reset viewports.
+    def page_setup(
+        self,
+        size: Tuple[float, float] = (297, 210),
+        margins: Tuple[float, float, float, float] = (10, 15, 10, 15),
+        units: str = "mm",
+        offset: Tuple[float, float] = (0, 0),
+        rotation: int = 0,
+        scale: Union[int, Tuple[float, float]] = 16,
+        name: str = "ezdxf",
+        device: str = "DWG to PDF.pc3",
+    ) -> None:
+        """Setup plot settings and paper size and reset viewports.
         All parameters in given `units` (mm or inch).
 
-        Reset paper limits, extends and viewports.
+        Reset paper limits, extents and viewports.
 
         Args:
             size: paper size as (width, height) tuple
@@ -555,27 +613,29 @@ class Paperspace(Layout):
 
         if isinstance(scale, tuple):
             standard_scale = 16
+            scale_num, scale_denom = scale
         elif isinstance(scale, int):
             standard_scale = scale
-            scale = const.STD_SCALES.get(standard_scale, (1, 1))
+            scale_num, scale_denom = const.STD_SCALES.get(
+                standard_scale, (1.0, 1.0)
+            )
         else:
             raise const.DXFTypeError(
-                "Scale has to be an int or a tuple(numerator, denominator)")
-        if scale[0] == 0:
+                "Scale has to be an int or a tuple(numerator, denominator)"
+            )
+        if scale_num == 0:
             raise const.DXFValueError("Scale numerator can't be 0.")
-        if scale[1] == 0:
+        if scale_denom == 0:
             raise const.DXFValueError("Scale denominator can't be 0.")
-
-        self.use_standard_scale(False)  # works best, don't know why
         paper_width, paper_height = size
         margin_top, margin_right, margin_bottom, margin_left = margins
         units = units.lower()
-        if units.startswith('inch'):
-            units = 'Inches'
+        if units.startswith("inch"):
+            units = "Inches"
             plot_paper_units = 0
             unit_factor = 25.4  # inch to mm
-        elif units == 'mm':
-            units = 'MM'
+        elif units == "mm":
+            units = "MM"
             plot_paper_units = 1
             unit_factor = 1.0
         else:
@@ -584,17 +644,22 @@ class Paperspace(Layout):
         # Setup PLOTSETTINGS
         # all paper sizes in mm
         dxf = self.dxf_layout.dxf
-        dxf.page_setup_name = ''
+        if not dxf.hasattr("plot_layout_flags"):
+            dxf.plot_layout_flags = dxf.get_default("plot_layout_flags")
+        self.use_standard_scale(False)  # works best, don't know why
+        dxf.page_setup_name = ""
         dxf.plot_configuration_file = device
-        dxf.paper_size = f'{name}_({paper_width:.2f}_x_{paper_height:.2f}_{units})'
+        dxf.paper_size = (
+            f"{name}_({paper_width:.2f}_x_{paper_height:.2f}_{units})"
+        )
         dxf.left_margin = margin_left * unit_factor
         dxf.bottom_margin = margin_bottom * unit_factor
         dxf.right_margin = margin_right * unit_factor
         dxf.top_margin = margin_top * unit_factor
         dxf.paper_width = paper_width * unit_factor
         dxf.paper_height = paper_height * unit_factor
-        dxf.scale_numerator = scale[0]
-        dxf.scale_denominator = scale[1]
+        dxf.scale_numerator = scale_num
+        dxf.scale_denominator = scale_denom
         dxf.plot_paper_units = plot_paper_units
         dxf.plot_rotation = rotation
 
@@ -602,15 +667,15 @@ class Paperspace(Layout):
         dxf.plot_origin_x_offset = x_offset * unit_factor  # conversion to mm
         dxf.plot_origin_y_offset = y_offset * unit_factor  # conversion to mm
         dxf.standard_scale_type = standard_scale
-        dxf.unit_factor = 1. / unit_factor  # 1/1 for mm; 1/25.4 ... for inch
+        dxf.unit_factor = 1.0 / unit_factor  # 1/1 for mm; 1/25.4 ... for inch
 
         # Setup Layout
         self.reset_paper_limits()
-        self.reset_extends()
+        self.reset_extents()
         self.reset_viewports()
 
     def reset_paper_limits(self) -> None:
-        """ Set paper limits to default values, all values in paperspace units
+        """Set paper limits to default values, all values in paperspace units
         but without plot scale (?).
 
         """
@@ -634,25 +699,27 @@ class Paperspace(Layout):
         dxf.limmin = (-shift_x, -shift_y)  # paper space units
         dxf.limmax = (paper_width - shift_x, paper_height - shift_y)
 
-    def get_paper_limits(self) -> Tuple[
-        Tuple[float, float], Tuple[float, float]]:
-        """  Returns paper limits in plot paper units, relative to the plot origin.
+    def get_paper_limits(self) -> Tuple[Vec2, Vec2]:
+        """Returns paper limits in plot paper units, relative to the plot origin.
 
         plot origin = lower left corner of printable area + plot origin offset
 
         Returns:
-            tuple ((x1, y1), (x2, y2)), lower left corner is (x1, y1), upper
-            right corner is (x2, y2).
+            tuple (Vec2(x1, y1), Vec2(x2, y2)), lower left corner is (x1, y1),
+            upper right corner is (x2, y2).
 
         """
-        return self.dxf.limmin[0:2], self.dxf.limmax[0:2]
+        return Vec2(self.dxf.limmin), Vec2(self.dxf.limmax)
 
-    def page_setup_r12(self, size: Tuple[int, int] = (297, 210),
-                       margins: Tuple[int, int, int, int] = (0, 0, 0, 0),
-                       units: str = 'mm',
-                       offset: Tuple[int, int] = (0, 0),
-                       rotation: float = 0,
-                       scale: int = 16) -> None:
+    def page_setup_r12(
+        self,
+        size: Tuple[int, int] = (297, 210),
+        margins: Tuple[int, int, int, int] = (0, 0, 0, 0),
+        units: str = "mm",
+        offset: Tuple[int, int] = (0, 0),
+        rotation: float = 0,
+        scale: Union[int, Tuple[float, float]] = 16,
+    ) -> None:
 
         # remove existing viewports
         for viewport in self.viewports():
@@ -662,23 +729,25 @@ class Paperspace(Layout):
             raise const.DXFValueError("Valid rotation values: 0-3")
 
         if isinstance(scale, int):
-            scale = const.STD_SCALES.get(scale, (1, 1))
+            scale_num, scale_denom = const.STD_SCALES.get(scale, (1, 1))
+        else:
+            scale_num, scale_denom = scale
 
-        if scale[0] == 0:
+        if scale_num == 0:
             raise const.DXFValueError("Scale numerator can't be 0.")
-        if scale[1] == 0:
+        if scale_denom == 0:
             raise const.DXFValueError("Scale denominator can't be 0.")
 
-        scale_factor = scale[1] / scale[0]
+        scale_factor = scale_denom / scale_num
 
         # TODO: don't know how to set inch or mm mode in R12
         units = units.lower()
-        if units.startswith('inch'):
-            units = 'Inches'
+        if units.startswith("inch"):
+            units = "Inches"
             plot_paper_units = 0
             unit_factor = 25.4  # inch to mm
-        elif units == 'mm':
-            units = 'MM'
+        elif units == "mm":
+            units = "MM"
             plot_paper_units = 1
             unit_factor = 1.0
         else:
@@ -697,19 +766,21 @@ class Paperspace(Layout):
         paper_width = paper_units(size[0])
         paper_height = paper_units(size[1])
 
-        plimmin = self.doc.header['$PLIMMIN'] = (0, 0)
-        plimmax = self.doc.header['$PLIMMAX'] = (paper_width, paper_height)
+        plimmin = self.doc.header["$PLIMMIN"] = (0, 0)
+        plimmax = self.doc.header["$PLIMMAX"] = (paper_width, paper_height)
 
         # TODO: don't know how paper setup in DXF R12 works
 
-        pextmin = self.doc.header['$PEXTMIN'] = (0, 0, 0)
-        pextmax = self.doc.header['$PEXTMAX'] = (paper_width, paper_height, 0)
+        pextmin = self.doc.header["$PEXTMIN"] = (0, 0, 0)
+        pextmax = self.doc.header["$PEXTMAX"] = (paper_width, paper_height, 0)
 
         # printing area
-        printable_width = paper_width - paper_units(margin_left) - paper_units(
-            margin_right)
-        printable_height = paper_height - paper_units(
-            margin_bottom) - paper_units(margin_top)
+        printable_width = (
+            paper_width - paper_units(margin_left) - paper_units(margin_right)
+        )
+        printable_height = (
+            paper_height - paper_units(margin_bottom) - paper_units(margin_top)
+        )
 
         # AutoCAD viewport (window) size
         vp_width = paper_width * 1.1
@@ -727,11 +798,10 @@ class Paperspace(Layout):
         )
         main_viewport.dxf.id = 1  # set as main viewport
         main_viewport.dxf.status = 2  # AutoCAD default value
-        with main_viewport.edit_data() as vpdata:
-            vpdata.view_mode = 1000  # AutoDesk default
+        main_viewport.dxf.render_mode = 1000  # AutoDesk default (view mode?)
 
-    def get_paper_limits_r12(self) -> Tuple[float, float]:
-        """  Returns paper limits in plot paper units. """
-        limmin = self.doc.header.get('$PLIMMIN', (0, 0))
-        limmax = self.doc.header.get('$PLIMMAX', (0, 0))
-        return limmin, limmax
+    def get_paper_limits_r12(self) -> Tuple[Vec2, Vec2]:
+        """Returns paper limits in plot paper units."""
+        limmin = self.doc.header.get("$PLIMMIN", (0, 0))
+        limmax = self.doc.header.get("$PLIMMAX", (0, 0))
+        return Vec2(limmin), Vec2(limmax)
