@@ -1,10 +1,22 @@
-#  Copyright (c) 2020, Manfred Moitzi
+#  Copyright (c) 2020-2022, Manfred Moitzi
 #  License: MIT License
+from __future__ import annotations
+import typing
 from typing import (
-    TYPE_CHECKING, BinaryIO, Iterable, List, Callable, Tuple,
+    TYPE_CHECKING,
+    BinaryIO,
+    Iterable,
+    List,
+    Callable,
+    Tuple,
+    Dict,
+    Union,
 )
 import itertools
+import re
 from collections import defaultdict
+from pathlib import Path
+import logging
 
 from ezdxf.lldxf import const
 from ezdxf.lldxf import repair
@@ -13,7 +25,12 @@ from ezdxf.lldxf.encoding import (
     decode_dxf_unicode,
 )
 from ezdxf.lldxf.types import (
-    DXFTag, DXFVertex, DXFBinaryTag, POINT_CODES, BINARY_DATA, TYPE_TABLE,
+    DXFTag,
+    DXFVertex,
+    DXFBinaryTag,
+    POINT_CODES,
+    BINARY_DATA,
+    TYPE_TABLE,
     MAX_GROUP_CODE,
 )
 from ezdxf.lldxf.tags import group_tags, Tags
@@ -24,16 +41,24 @@ from ezdxf.audit import Auditor, AuditError
 if TYPE_CHECKING:
     from ezdxf.eztypes import Drawing, SectionDict
 
-__all__ = ['read', 'readfile']
+__all__ = ["read", "readfile"]
 
 EXCLUDE_STRUCTURE_CHECK = {
-    'SECTION', 'ENDSEC', 'EOF', 'TABLE', 'ENDTAB', 'ENDBLK', 'SEQEND'
+    "SECTION",
+    "ENDSEC",
+    "EOF",
+    "TABLE",
+    "ENDTAB",
+    "ENDBLK",
+    "SEQEND",
 }
+logger = logging.getLogger("ezdxf")
 
 
-def readfile(filename: str,
-             errors: str = 'surrogateescape') -> Tuple['Drawing', 'Auditor']:
-    """ Read a DXF document from file system similar to :func:`ezdxf.readfile`,
+def readfile(
+    filename: Union[str, Path], errors: str = "surrogateescape"
+) -> Tuple["Drawing", "Auditor"]:
+    """Read a DXF document from file system similar to :func:`ezdxf.readfile`,
     but this function will repair as much flaws as possible, runs the required
     audit process automatically the DXF document and the :class:`Auditor`.
 
@@ -50,15 +75,17 @@ def readfile(filename: str,
         UnicodeDecodeError: if `errors` is "strict" and a decoding error occurs
 
     """
-    with open(filename, mode='rb') as fp:
+    filename = str(filename)
+    with open(filename, mode="rb") as fp:
         doc, auditor = read(fp, errors=errors)
     doc.filename = filename
     return doc, auditor
 
 
-def read(stream: BinaryIO,
-         errors: str = 'surrogateescape') -> Tuple['Drawing', 'Auditor']:
-    """ Read a DXF document from a binary-stream similar to :func:`ezdxf.read`,
+def read(
+    stream: BinaryIO, errors: str = "surrogateescape"
+) -> Tuple["Drawing", "Auditor"]:
+    """Read a DXF document from a binary-stream similar to :func:`ezdxf.read`,
     but this function will detect the text encoding automatically and repair
     as much flaws as possible, runs the required audit process afterwards
     and returns the DXF document and the :class:`Auditor`.
@@ -76,8 +103,45 @@ def read(stream: BinaryIO,
         UnicodeDecodeError: if `errors` is "strict" and a decoding error occurs
 
     """
-    from ezdxf.document import Drawing
     recover_tool = Recover.run(stream, errors=errors)
+    return _load_and_audit_document(recover_tool)
+
+
+def explore(
+    filename: Union[str, Path], errors: str = "ignore"
+) -> Tuple["Drawing", "Auditor"]:
+    """Read a DXF document from file system similar to :func:`readfile`,
+    but this function will use a special tag loader, which synchronise the tag
+    stream if invalid tags occur. This function is intended to load corrupted
+    DXF files and should only be used to explore such files, data loss is very
+    likely.
+
+    Args:
+        filename: file-system name of the DXF document to load
+        errors: specify decoding error handler
+
+            - "surrogateescape" to preserve possible binary data (default)
+            - "ignore" to use the replacement char U+FFFD "\ufffd" for invalid data
+            - "strict" to raise an :class:`UnicodeDecodeError` exception for invalid data
+
+    Raises:
+        DXFStructureError: for invalid or corrupted DXF structures
+        UnicodeDecodeError: if `errors` is "strict" and a decoding error occurs
+
+    """
+    filename = str(filename)
+    with open(filename, mode="rb") as fp:
+        recover_tool = Recover.run(
+            fp, errors=errors, loader=synced_bytes_loader
+        )
+        doc, auditor = _load_and_audit_document(recover_tool)
+    doc.filename = filename
+    return doc, auditor
+
+
+def _load_and_audit_document(recover_tool) -> Tuple["Drawing", "Auditor"]:
+    from ezdxf.document import Drawing
+
     doc = Drawing()
     doc._load_section_dict(recover_tool.section_dict)
 
@@ -92,7 +156,7 @@ def read(stream: BinaryIO,
 
 # noinspection PyMethodMayBeStatic
 class Recover:
-    """ Loose coupled recovering tools. """
+    """Loose coupled recovering tools."""
 
     def __init__(self, loader: Callable = None):
         # different tag loading strategies can be used:
@@ -102,38 +166,49 @@ class Recover:
         self.tag_loader = loader or bytes_loader
 
         # The main goal of all efforts, a Drawing compatible dict of sections:
-        self.section_dict: 'SectionDict' = dict()
+        self.section_dict: "SectionDict" = dict()
 
         # Store error messages from low level processes
-        self.errors = []
-        self.fixes = []
+        self.errors: List[Tuple[int, str]] = []
+        self.fixes: List[Tuple[int, str]] = []
+
+        # Detected DXF version
+        self.dxfversion = const.DXF12
 
     @classmethod
-    def run(cls, stream: BinaryIO, loader: Callable = None,
-            errors: str = 'surrogateescape') -> 'Recover':
-        """ Execute the recover process. """
+    def run(
+        cls,
+        stream: BinaryIO,
+        loader: Callable = None,
+        errors: str = "surrogateescape",
+    ) -> "Recover":
+        """Execute the recover process."""
         recover_tool = Recover(loader)
         tags = recover_tool.load_tags(stream, errors)
         sections = recover_tool.rebuild_sections(tags)
         recover_tool.load_section_dict(sections)
-        tables = recover_tool.section_dict.get('TABLES')
+        tables = recover_tool.section_dict.get("TABLES")
         if tables:
-            tables = recover_tool.rebuild_tables(tables)
-            recover_tool.section_dict['TABLES'] = tables
-
+            tables = recover_tool.rebuild_tables(tables)  # type: ignore
+            recover_tool.section_dict["TABLES"] = tables
+        if recover_tool.dxfversion > "AC1009":
+            recover_tool.recover_rootdict()
         section_dict = recover_tool.section_dict
         for name, entities in section_dict.items():
-            if name in {'TABLES', 'BLOCKS', 'OBJECTS', 'ENTITIES'}:
-                section_dict[name] = list(recover_tool.check_entities(entities))
+            if name in {"TABLES", "BLOCKS", "OBJECTS", "ENTITIES"}:
+                section_dict[name] = list(
+                    recover_tool.check_entities(entities)  # type: ignore
+                )
 
         return recover_tool
 
     def load_tags(self, stream: BinaryIO, errors: str) -> Iterable[DXFTag]:
-        return safe_tag_loader(stream, self.tag_loader,
-                               messages=self.errors, errors=errors)
+        return safe_tag_loader(
+            stream, self.tag_loader, messages=self.errors, errors=errors
+        )
 
     def rebuild_sections(self, tags: Iterable[DXFTag]) -> List[List[DXFTag]]:
-        """ Collect tags between SECTION and ENDSEC or next SECTION tag
+        """Collect tags between SECTION and ENDSEC or next SECTION tag
         as list of DXFTag objects, collects tags outside of sections
         as an extra section.
 
@@ -143,6 +218,7 @@ class Recover:
 
         """
 
+        # Invalid placed DXF entities are removed in the audit process!
         def close_section():
             # ENDSEC tag is not collected
             nonlocal collector, inside_section
@@ -150,35 +226,41 @@ class Recover:
                 sections.append(collector)
             else:  # missing SECTION
                 # ignore this tag, it is even not an orphan
-                self.fixes.append((
-                    AuditError.MISSING_SECTION_TAG,
-                    'DXF structure error: missing SECTION tag.'
-                ))
+                self.fixes.append(
+                    (
+                        AuditError.MISSING_SECTION_TAG,
+                        "DXF structure error: missing SECTION tag.",
+                    )
+                )
             collector = []
             inside_section = False
 
         def open_section():
             nonlocal inside_section
             if inside_section:  # missing ENDSEC
-                self.fixes.append((
-                    AuditError.MISSING_ENDSEC_TAG,
-                    'DXF structure error: missing ENDSEC tag.'
-                ))
+                self.fixes.append(
+                    (
+                        AuditError.MISSING_ENDSEC_TAG,
+                        "DXF structure error: missing ENDSEC tag.",
+                    )
+                )
                 close_section()
             collector.append(tag)
             inside_section = True
 
         def process_structure_tag():
-            if value == 'SECTION':
+            if value == "SECTION":
                 open_section()
-            elif value == 'ENDSEC':
+            elif value == "ENDSEC":
                 close_section()
-            elif value == 'EOF':
+            elif value == "EOF":
                 if inside_section:
-                    self.fixes.append((
-                        AuditError.MISSING_ENDSEC_TAG,
-                        'DXF structure error: missing ENDSEC tag.'
-                    ))
+                    self.fixes.append(
+                        (
+                            AuditError.MISSING_ENDSEC_TAG,
+                            "DXF structure error: missing ENDSEC tag.",
+                        )
+                    )
                     close_section()
             else:
                 collect()
@@ -187,16 +269,18 @@ class Recover:
             if inside_section:
                 collector.append(tag)
             else:
-                self.fixes.append((
-                    AuditError.FOUND_TAG_OUTSIDE_SECTION,
-                    f'DXF structure error: found tag outside section: '
-                    f'({code}, {value})'
-                ))
+                self.fixes.append(
+                    (
+                        AuditError.FOUND_TAG_OUTSIDE_SECTION,
+                        f"DXF structure error: found tag outside section: "
+                        f"({code}, {value})",
+                    )
+                )
                 orphans.append(tag)
 
-        orphans = []
-        sections = []
-        collector = []
+        orphans: List[DXFTag] = []
+        sections: List[List[DXFTag]] = []
+        collector: List[DXFTag] = []
         inside_section = False
         for tag in tags:
             code, value = tag
@@ -209,7 +293,7 @@ class Recover:
         return sections
 
     def load_section_dict(self, sections: List[List[DXFTag]]) -> None:
-        """ Merge sections of same type. """
+        """Merge sections of same type."""
 
         def add_section(name: str, tags) -> None:
             if name in section_dict:
@@ -217,38 +301,61 @@ class Recover:
             else:
                 section_dict[name] = tags
 
-        def _build_section_dict(d: dict) -> None:
+        def _build_section_dict(d: Dict) -> None:
             for name, section in d.items():
                 if name in const.MANAGED_SECTIONS:
                     self.section_dict[name] = list(group_tags(section, 0))
 
+        def _remove_unsupported_sections(d: Dict):
+            for name in ("CLASSES", "OBJECTS", "ACDSDATA"):
+                if name in d:
+                    del d[name]
+                    self.fixes.append(
+                        (
+                            AuditError.REMOVED_UNSUPPORTED_SECTION,
+                            f"Removed unsupported {name} section for DXF R12.",
+                        )
+                    )
+
         # Last section could be orphaned tags:
         orphans = sections.pop()
-        if orphans and orphans[0] == (0, 'SECTION'):
+        if orphans and orphans[0] == (0, "SECTION"):
             # The last section contains not the orphaned tags:
             sections.append(orphans)
             orphans = []
 
-        section_dict = dict()
+        section_dict: "SectionDict" = dict()
         for section in sections:
             code, name = section[1]
             if code == 2:
                 add_section(name, section)
             else:  # invalid section name tag e.g. (2, "HEADER")
-                self.fixes.append((
-                    AuditError.MISSING_SECTION_NAME_TAG,
-                    'DXF structure error: missing section name tag, ignore section.'
-                ))
+                self.fixes.append(
+                    (
+                        AuditError.MISSING_SECTION_NAME_TAG,
+                        "DXF structure error: missing section name tag, ignore section.",
+                    )
+                )
 
-        header = section_dict.setdefault('HEADER', [
-            DXFTag(0, 'SECTION'),
-            DXFTag(2, 'HEADER'),
-        ])
-        self.rescue_orphaned_header_vars(header, orphans)
+        header = section_dict.setdefault(
+            "HEADER",
+            [
+                DXFTag(0, "SECTION"),  # type: ignore
+                DXFTag(2, "HEADER"),  # type: ignore
+            ],
+        )
+        self.rescue_orphaned_header_vars(header, orphans)  # type: ignore
+        self.dxfversion = _detect_dxf_version(header)
+        if self.dxfversion <= const.DXF12:
+            _remove_unsupported_sections(section_dict)
         _build_section_dict(section_dict)
 
     def rebuild_tables(self, tables: List[Tags]) -> List[Tags]:
-        """ Rebuild TABLES section. """
+        """Rebuild TABLES section."""
+
+        # Note: the recover module does not report invalid placed table entries,
+        # it just recovers them. The "normal" loading process ignore these
+        # misplaced table entries and logs a warning.
 
         def append_table(name: str):
             if name not in content:
@@ -259,9 +366,9 @@ class Recover:
                 tables.append(head)
             else:
                 # The new table head gets a valid handle from Auditor.
-                tables.append(Tags([DXFTag(0, 'TABLE'), DXFTag(2, name)]))
+                tables.append(Tags([DXFTag(0, "TABLE"), DXFTag(2, name)]))
             tables.extend(content[name])
-            tables.append(Tags([DXFTag(0, 'ENDTAB')]))
+            tables.append(Tags([DXFTag(0, "ENDTAB")]))
 
         heads = dict()
         content = defaultdict(list)
@@ -269,7 +376,7 @@ class Recover:
 
         for entry in tables:
             name = entry[0].value.upper()
-            if name == 'TABLE':
+            if name == "TABLE":
                 try:
                     table_name = entry[1].value.upper()
                 except (IndexError, AttributeError):
@@ -278,15 +385,27 @@ class Recover:
                     heads[table_name] = entry
             elif name in valid_tables:
                 content[name].append(entry)
-        tables = [Tags([DXFTag(0, 'SECTION'), DXFTag(2, 'TABLES')])]
-        for name in const.TABLE_NAMES_ACAD_ORDER:
+        tables = [Tags([DXFTag(0, "SECTION"), DXFTag(2, "TABLES")])]
+
+        names = list(const.TABLE_NAMES_ACAD_ORDER)
+        if self.dxfversion <= const.DXF12:
+            # Ignore BLOCK_RECORD table
+            names.remove("BLOCK_RECORD")
+            if "BLOCK_RECORD" in content:
+                self.fixes.append(
+                    (
+                        AuditError.REMOVED_UNSUPPORTED_TABLE,
+                        f"Removed unsupported BLOCK_RECORD table for DXF R12.",
+                    )
+                )
+
+        for name in names:
             append_table(name)
         return tables
 
     def rescue_orphaned_header_vars(
-            self,
-            header: List[DXFTag],
-            orphans: Iterable[DXFTag]) -> None:
+        self, header: List[DXFTag], orphans: Iterable[DXFTag]
+    ) -> None:
         var_name = None
         for tag in orphans:
             code, value = tag
@@ -306,13 +425,65 @@ class Recover:
                 # raises DXFStructureError() for invalid entities
                 yield Tags(entity_structure_validator(entity))
 
+    def recover_rootdict(self):
+        objects = self.section_dict.get("OBJECTS")
+        if not objects or len(objects) < 2:
+            return  # empty OBJECTS section
+        # index 0 is [DXFTag(0, 'SECTION'), DXFTag(2, 'OBJECTS')], this is a
+        # requirement to be stored in the section_dict!
+        if _is_rootdict(objects[1]):
+            return  # everything is fine
+        index, rootdict = _find_rootdict(objects)
+        if index:  # make rootdict to first entity in OBJECTS section
+            objects[index] = objects[1]
+            objects[1] = rootdict
+            try:
+                handle = rootdict.get_handle()
+            except const.DXFValueError:
+                handle = "None"
+            self.fixes.append(
+                (
+                    AuditError.MISPLACED_ROOT_DICT,
+                    f"Recovered misplaced root DICTIONARY(#{handle}).",
+                )
+            )
 
-def safe_tag_loader(stream: BinaryIO,
-                    loader: Callable = None,
-                    messages: List = None,
-                    errors: str = 'surrogateescape',
-                    ) -> Iterable[DXFTag]:
-    """ Yields :class:``DXFTag`` objects from a bytes `stream`
+
+def _detect_dxf_version(header: List) -> str:
+    next_is_dxf_version = False
+    for tag in header:
+        if next_is_dxf_version:
+            dxfversion = str(tag[1]).strip()
+            if re.fullmatch(r"AC[0-9]{4}", dxfversion):
+                return dxfversion
+            else:
+                break
+        if tag == (9, "$ACADVER"):
+            next_is_dxf_version = True
+    return const.DXF12
+
+
+def _is_rootdict(entity: Tags) -> bool:
+    if entity[0] != (0, "DICTIONARY"):
+        return False
+    # The entry "ACAD_GROUP" in the rootdict is absolutely necessary!
+    return any(tag == (3, "ACAD_GROUP") for tag in entity)
+
+
+def _find_rootdict(objects: List[Tags]) -> Tuple[int, Tags]:
+    for index, entity in enumerate(objects):
+        if _is_rootdict(entity):
+            return index, entity
+    return 0, Tags()
+
+
+def safe_tag_loader(
+    stream: BinaryIO,
+    loader: Callable = None,
+    messages: List = None,
+    errors: str = "surrogateescape",
+) -> Iterable[DXFTag]:
+    """Yields :class:``DXFTag`` objects from a bytes `stream`
     (untrusted external  source), skips all comment tags (group code == 999).
 
     - Fixes unordered and invalid vertex tags.
@@ -336,13 +507,51 @@ def safe_tag_loader(stream: BinaryIO,
     encoding = detect_encoding(detector_stream)
 
     # Apply repair filter:
-    tags = repair.tag_reorder_layer(tags)
-    tags = repair.filter_invalid_yz_point_codes(tags)
+    tags = repair.tag_reorder_layer(tags)  # type: ignore
+    tags = repair.filter_invalid_point_codes(tags)  # type: ignore
+    tags = repair.filter_invalid_handles(tags)  # type: ignore
     return byte_tag_compiler(tags, encoding, messages=messages, errors=errors)
 
 
+INT_PATTERN_S = re.compile(r"[+-]?\d+")
+INT_PATTERN_B = re.compile(rb"[+-]?\d+")
+
+
+def _search_int(s: Union[str, bytes]) -> int:
+    """Emulate the behavior of the C function stoll(), which just stop
+    converting strings to integers at the first invalid char without raising
+    an exception. e.g. "42xyz" is a valid integer 42
+
+    """
+    res = re.search(  # type: ignore
+        INT_PATTERN_S if isinstance(s, str) else INT_PATTERN_B, s
+    )
+    if res:
+        s = res.group()
+    return int(s)
+
+
+FLOAT_PATTERN_S = re.compile(r"[+-]?\d+(:?\.\d*)?(:?[eE][+-]?\d+)?")
+FLOAT_PATTERN_B = re.compile(rb"[+-]?\d+(:?\.\d*)?(:?[eE][+-]?\d+)?")
+
+
+def _search_float(s: Union[str, bytes]) -> float:
+    """Emulate the behavior of the C function stod(), which just stop
+    converting strings to doubles at the first invalid char without raising
+    an exception. e.g. "47.11xyz" is a valid double 47.11
+
+    """
+    res = re.search(  # type: ignore
+        FLOAT_PATTERN_S if isinstance(s, str) else FLOAT_PATTERN_B, s
+    )
+    if res:
+        s = res.group()
+    return float(s)
+
+
+@typing.no_type_check
 def bytes_loader(stream: BinaryIO) -> Iterable[DXFTag]:
-    """ Yields :class:``DXFTag`` objects from a bytes `stream`
+    """Yields :class:``DXFTag`` objects from a bytes `stream`
     (untrusted external  source), skips all comment tags (group code == 999).
 
     ``DXFTag.code`` is always an ``int`` and ``DXFTag.value`` is always a
@@ -353,34 +562,41 @@ def bytes_loader(stream: BinaryIO) -> Iterable[DXFTag]:
         DXFStructureError: Found invalid group code.
 
     """
+    eof = False
     line = 1
-    while True:
-        try:
-            code = stream.readline()
-            value = stream.readline()
-        except EOFError:
-            # EOFError indicates a DXFStructureError, but should be handled
-            # in top layers.
-            return
-
-        # ByteIO(): empty strings indicates EOF
-        if code and value:
+    readline = stream.readline
+    while not eof:
+        code = readline()
+        # ByteIO(): empty strings indicates EOF - does not raise an exception
+        if code:
             try:
                 code = int(code)
             except ValueError:
-                code = code.decode(const.DEFAULT_ENCODING)
-                raise const.DXFStructureError(
-                    f'Invalid group code "{code}" at line {line}.')
-            else:
-                if code != 999:
-                    yield DXFTag(code, value.rstrip(b'\r\n'))
-                line += 2
+                try:  # harder to find an int
+                    code = _search_int(code)
+                except ValueError:
+                    code = code.decode(errors="ignore").rstrip("\r\n")
+                    raise const.DXFStructureError(
+                        f'Invalid group code "{code}" at line {line}.'
+                    )
+        else:
+            return
+
+        value = readline()
+        # ByteIO(): empty strings indicates EOF
+        if value:
+            value = value.rstrip(b"\r\n")
+            if code == 0 and value == b"EOF":
+                eof = True
+            if code != 999:
+                yield DXFTag(code, value)
+            line += 2
         else:
             return
 
 
 def synced_bytes_loader(stream: BinaryIO) -> Iterable[DXFTag]:
-    """ Yields :class:``DXFTag`` objects from a bytes `stream`
+    """Yields :class:``DXFTag`` objects from a bytes `stream`
     (untrusted external source), skips all comment tags (group code == 999).
 
     ``DXFTag.code`` is always an ``int`` and ``DXFTag.value`` is always a
@@ -396,35 +612,35 @@ def synced_bytes_loader(stream: BinaryIO) -> Iterable[DXFTag]:
     """
     code = 999
     upper_boundary = MAX_GROUP_CODE + 1
+    readline = stream.readline
     while True:
         seeking_valid_group_code = True
-        try:
-            while seeking_valid_group_code:
-                code = stream.readline()
-                if code:
-                    try:
-                        code = int(code)
-                    except ValueError:
-                        pass
-                    else:
-                        if 0 <= code < upper_boundary:
-                            seeking_valid_group_code = False
+        while seeking_valid_group_code:
+            code = readline()  # type: ignore
+            if code:
+                try:  # hard to find an int
+                    code = _search_int(code)  # type: ignore
+                except ValueError:
+                    pass
                 else:
-                    return  # total empty result is EOF
-            value = stream.readline()
-        except EOFError:
-            return
-
-        if code != 999:
-            yield DXFTag(code, value.rstrip(b'\r\n'))
+                    if 0 <= code < upper_boundary:
+                        seeking_valid_group_code = False
+            else:
+                return  # empty string is EOF
+        value = readline()
+        if value:
+            if code != 999:
+                yield DXFTag(code, value.rstrip(b"\r\n"))
+        else:
+            return  # empty string is EOF
 
 
-DWGCODEPAGE = b'$DWGCODEPAGE'
-ACADVER = b'$ACADVER'
+DWGCODEPAGE = b"$DWGCODEPAGE"
+ACADVER = b"$ACADVER"
 
 
 def detect_encoding(tags: Iterable[DXFTag]) -> str:
-    """ Detect text encoding from header variables $DWGCODEPAGE and $ACADVER
+    """Detect text encoding from header variables $DWGCODEPAGE and $ACADVER
     out of a stream of DXFTag objects.
 
     Assuming a malformed DXF file:
@@ -455,17 +671,19 @@ def detect_encoding(tags: Iterable[DXFTag]) -> str:
             next_tag = None
 
         if encoding and dxfversion:
-            return 'utf8' if dxfversion >= const.DXF2007 else encoding
+            return "utf8" if dxfversion >= const.DXF2007 else encoding
 
     return const.DEFAULT_ENCODING
 
 
-def byte_tag_compiler(tags: Iterable[DXFTag],
-                      encoding=const.DEFAULT_ENCODING,
-                      messages: List = None,
-                      errors: str = 'surrogateescape',
-                      ) -> Iterable[DXFTag]:
-    """ Compiles DXF tag values imported by bytes_loader() into Python types.
+@typing.no_type_check
+def byte_tag_compiler(
+    tags: Iterable[DXFTag],
+    encoding=const.DEFAULT_ENCODING,
+    messages: List = None,
+    errors: str = "surrogateescape",
+) -> Iterable[DXFTag]:
+    """Compiles DXF tag values imported by bytes_loader() into Python types.
 
     Raises DXFStructureError() for invalid float values and invalid coordinate
     values.
@@ -493,6 +711,27 @@ def byte_tag_compiler(tags: Iterable[DXFTag],
         value = tag.value.decode(encoding)
         return f'Invalid tag ({code}, "{value}") near line: {line}.'
 
+    def recover_int(s: Union[str, bytes]) -> int:
+        if isinstance(s, bytes):
+            s = s.decode(encoding="utf8", errors="ignore")
+        value = _search_int(s)
+        msg = f'recovered invalid integer value "{s}" near line {line} as "{value}"'
+        messages.append((AuditError.INVALID_INTEGER_VALUE, msg))
+        logger.warning(msg)
+        return value
+
+    def recover_float(s: Union[str, bytes]) -> float:
+        if isinstance(s, bytes):
+            s = s.decode(encoding="utf8", errors="ignore")
+        value = _search_float(s)
+        msg = f'recovered invalid floating point value "{s}" near line {line} as "{value}"'
+        messages.append((AuditError.INVALID_FLOATING_POINT_VALUE, msg))
+        logger.warning(msg)
+        return value
+
+    assert isinstance(encoding, str)
+    assert isinstance(errors, str)
+
     if messages is None:
         messages = []
     tags = iter(tags)
@@ -513,20 +752,39 @@ def byte_tag_compiler(tags: Iterable[DXFTag],
                 # e.g. y-code for x-code=10 is 20
                 if y.code != code + 10:
                     raise const.DXFStructureError(
-                        f"Missing required y-coordinate near line: {line}.")
+                        f"Missing required y-coordinate near line: {line}."
+                    )
                 # optional z coordinate
                 z = next(tags)
                 line += 2
                 try:
                     # is it a z-coordinate like (30, 0.0) for base x-code=10
                     if z.code == code + 20:
-                        point = (float(x.value), float(y.value), float(z.value))
+                        try:
+                            point = (
+                                float(x.value),
+                                float(y.value),
+                                float(z.value),
+                            )
+                        except ValueError:  # search for any float values
+                            point = (
+                                recover_float(x.value),
+                                recover_float(y.value),
+                                recover_float(z.value),
+                            )
                     else:
-                        point = (float(x.value), float(y.value))
+                        try:
+                            point = (float(x.value), float(y.value))
+                        except ValueError:  # search for any float values
+                            point = (
+                                recover_float(x.value),
+                                recover_float(y.value),
+                            )
                         undo_tag = z
                 except ValueError:
                     raise const.DXFStructureError(
-                        f'Invalid floating point values near line: {line}.')
+                        f"Invalid floating point values near line: {line}."
+                    )
                 yield DXFVertex(code, point)
             elif code in BINARY_DATA:
                 # maybe pre compiled in low level tagger (binary DXF)
@@ -537,7 +795,8 @@ def byte_tag_compiler(tags: Iterable[DXFTag],
                         tag = DXFBinaryTag.from_string(code, x.value)
                     except ValueError:
                         raise const.DXFStructureError(
-                            f'Invalid binary data near line: {line}.')
+                            f"Invalid binary data near line: {line}."
+                        )
                 yield tag
             else:  # just a single tag
                 type_ = TYPE_TABLE.get(code, str)
@@ -547,13 +806,15 @@ def byte_tag_compiler(tags: Iterable[DXFTag],
                         # remove white space from structure tags
                         value = x.value.strip().upper()
                     try:  # 2 stages to document decoding errors
-                        str_ = value.decode(encoding, errors='strict')
+                        str_ = value.decode(encoding, errors="strict")
                     except UnicodeDecodeError:
                         str_ = value.decode(encoding, errors=errors)
-                        messages.append((
-                            AuditError.DECODING_ERROR,
-                            f'Fixed unicode decoding error near line {line}'
-                        ))
+                        messages.append(
+                            (
+                                AuditError.DECODING_ERROR,
+                                f"Fixed unicode decoding error near line {line}",
+                            )
+                        )
 
                     # Convert DXF unicode notation "\U+xxxx" to unicode,
                     # but exclude structure tags (code>0):
@@ -566,10 +827,15 @@ def byte_tag_compiler(tags: Iterable[DXFTag],
                         # fast path for int and float
                         yield DXFTag(code, type_(value))
                     except ValueError:
-                        # slow path - ProE stores int values as floats :((
+                        # slow path - e.g. ProE stores int values as floats :((
                         if type_ is int:
                             try:
-                                yield DXFTag(code, int(float(x.value)))
+                                yield DXFTag(code, recover_int(x.value))
+                            except ValueError:
+                                raise const.DXFStructureError(error_msg(x))
+                        elif type_ is float:
+                            try:
+                                yield DXFTag(code, recover_float(x.value))
                             except ValueError:
                                 raise const.DXFStructureError(error_msg(x))
                         else:
