@@ -1,29 +1,28 @@
-# Created: 03.05.2014
-# Copyright (c) 2014-2020, Manfred Moitzi
+# Copyright (c) 2014-2021, Manfred Moitzi
 # License: MIT License
-from typing import Iterable, Any, Sequence, Union, Tuple
+from typing import Iterable, Any, Sequence, Union, Tuple, overload, Optional
 from array import array
 import struct
-from binascii import unhexlify
+from binascii import unhexlify, hexlify
 from codecs import decode
 
 Bytes = Union[bytes, bytearray, memoryview]
 
 
 def hex_strings_to_bytes(data: Iterable[str]) -> bytes:
-    """ Returns multiple hex strings `data` as bytes. """
-    byte_array = array('B')
+    """Returns multiple hex strings `data` as bytes."""
+    byte_array = array("B")
     for hexstr in data:
         byte_array.extend(unhexlify(hexstr))
     return byte_array.tobytes()
 
 
 def bytes_to_hexstr(data: bytes) -> str:
-    """ Returns `data` bytes as plain hex string. """
-    return ''.join("%0.2X" % byte for byte in data)
+    """Returns `data` bytes as plain hex string."""
+    return hexlify(data).upper().decode()
 
 
-NULL_NULL = b'\x00\x00'
+NULL_NULL = b"\x00\x00"
 
 
 class EndOfBufferError(EOFError):
@@ -31,7 +30,8 @@ class EndOfBufferError(EOFError):
 
 
 class ByteStream:
-    """ Process little endian binary data organized as bytes, data is padded to 4 byte boundaries by default.
+    """Process little endian binary data organized as bytes, data is padded to
+    4 byte boundaries by default.
     """
 
     # Created for Proxy Entity Graphic decoding
@@ -49,58 +49,74 @@ class ByteStream:
         return index + self._align - modulo if modulo else index
 
     def read_struct(self, fmt: str) -> Any:
-        """ Read data defined by a struct format string. Insert little endian format character '<' as
-        first character, if machine has native big endian byte order.
+        """Read data defined by a struct format string. Insert little endian
+        format character '<' as first character, if machine has native big
+        endian byte order.
         """
         if not self.has_data:
-            raise EndOfBufferError('Unexpected end of buffer.')
+            raise EndOfBufferError("Unexpected end of buffer.")
 
         result = struct.unpack_from(fmt, self.buffer, offset=self.index)
         self.index = self.align(self.index + struct.calcsize(fmt))
         return result
 
-    def read_float(self):
-        return self.read_struct('<d')[0]
+    def read_float(self) -> float:
+        return self.read_struct("<d")[0]
 
-    def read_long(self):
-        return self.read_struct('<L')[0]
+    def read_long(self) -> int:
+        return self.read_struct("<L")[0]
 
-    def read_signed_long(self):
-        return self.read_struct('<l')[0]
+    def read_signed_long(self) -> int:
+        return self.read_struct("<l")[0]
 
-    def read_vertex(self):
-        return self.read_struct('<3d')
+    def read_vertex(self) -> Sequence[float]:
+        return self.read_struct("<3d")
 
-    def read_padded_string(self, encoding: str = 'utf_8') -> str:
-        """ PS: Padded String. This is a string, terminated with a zero byte. The file’s text encoding (code page)
-        is used to encode/decode the bytes into a string.
+    def read_padded_string(self, encoding: str = "utf_8") -> str:
+        """PS: Padded String. This is a string, terminated with a zero byte.
+        The file’s text encoding (code page) is used to encode/decode the bytes
+        into a string.
         """
         buffer = self.buffer
         for end_index in range(self.index, len(buffer)):
             if buffer[end_index] == 0:
                 start_index = self.index
                 self.index = self.align(end_index + 1)
+                # noinspection PyTypeChecker
                 return decode(buffer[start_index:end_index], encoding=encoding)
-        raise EndOfBufferError('Unexpected end of buffer, did not detect terminating zero byte.')
+        raise EndOfBufferError(
+            "Unexpected end of buffer, did not detect terminating zero byte."
+        )
 
     def read_padded_unicode_string(self) -> str:
-        """ PUS: Padded Unicode String. The bytes are encoded using Unicode encoding. The bytes consist of
-        byte pairs and the string is terminated by 2 zero bytes.
+        """PUS: Padded Unicode String. The bytes are encoded using Unicode
+        encoding. The bytes consist of byte pairs and the string is terminated
+        by 2 zero bytes.
         """
         buffer = self.buffer
         for end_index in range(self.index, len(buffer), 2):
-            if buffer[end_index:end_index + 2] == NULL_NULL:
+            if buffer[end_index: end_index + 2] == NULL_NULL:
                 start_index = self.index
                 self.index = self.align(end_index + 2)
-                return decode(buffer[start_index:end_index], encoding='utf_16_le')
-        raise EndOfBufferError('Unexpected end of buffer, did not detect terminating zero bytes.')
+                # noinspection PyTypeChecker
+                return decode(
+                    buffer[start_index:end_index], encoding="utf_16_le"
+                )
+        raise EndOfBufferError(
+            "Unexpected end of buffer, did not detect terminating zero bytes."
+        )
 
 
 class BitStream:
-    """ Process little endian binary data organized as bit stream. """
+    """Process little endian binary data organized as bit stream."""
 
-    # Created for DWG bit stream decoding
-    def __init__(self, buffer: Bytes, dxfversion: str = 'AC1015', encoding: str = 'cp1252'):
+    # Created for Proxy Entity Graphic decoding and DWG bit stream decoding
+    def __init__(
+        self,
+        buffer: Bytes,
+        dxfversion: str = "AC1015",
+        encoding: str = "cp1252",
+    ):
         self.buffer = memoryview(buffer)
         self.bit_index: int = 0
         self.dxfversion = dxfversion
@@ -111,7 +127,7 @@ class BitStream:
         return self.bit_index >> 3 < len(self.buffer)
 
     def align(self, count: int) -> None:
-        """ Align to byte border. """
+        """Align to byte border."""
         byte_index = (self.bit_index >> 3) + bool(self.bit_index & 7)
         modulo = byte_index % count
         if modulo:
@@ -119,26 +135,28 @@ class BitStream:
         self.bit_index = byte_index << 3
 
     def skip(self, count: int) -> None:
-        """ Skip `count` bits. """
+        """Skip `count` bits."""
         self.bit_index += count
 
     def read_bit(self) -> int:
-        """ Read one bit from buffer. """
+        """Read one bit from buffer."""
         index = self.bit_index
         self.bit_index += 1
         try:
             return 1 if self.buffer[index >> 3] & (0x80 >> (index & 7)) else 0
         except IndexError:
-            raise EndOfBufferError('Unexpected end of buffer.')
+            raise EndOfBufferError("Unexpected end of buffer.")
 
     def read_bits(self, count) -> int:
-        """ Read `count` bits from buffer. """
+        """Read `count` bits from buffer."""
         index = self.bit_index
         buffer = self.buffer
-        next_bit_index = index + count  # index of next bit after reading `count` bits
+        # index of next bit after reading `count` bits
+        next_bit_index = index + count
 
-        if (next_bit_index - 1) >> 3 > len(buffer):  # not enough data to read all bits
-            raise EndOfBufferError('Unexpected end of buffer.')
+        if (next_bit_index - 1) >> 3 > len(buffer):
+            # not enough data to read all bits
+            raise EndOfBufferError("Unexpected end of buffer.")
         self.bit_index = next_bit_index
 
         test_bit = 0x80 >> (index & 7)
@@ -158,15 +176,15 @@ class BitStream:
         return value
 
     def read_unsigned_byte(self) -> int:
-        """ Read an unsigned byte (8 bit) from buffer. """
+        """Read an unsigned byte (8 bit) from buffer."""
         return self.read_bits(8)
 
     def read_signed_byte(self) -> int:
-        """ Read a signed byte (8 bit) from buffer. """
+        """Read a signed byte (8 bit) from buffer."""
         value = self.read_bits(8)
         if value & 0x80:
             # 2er complement
-            return -((~value & 0xff) + 1)
+            return -((~value & 0xFF) + 1)
         else:
             return value
 
@@ -176,12 +194,12 @@ class BitStream:
         end_index = start_index + count
         if end_index <= len(buffer):
             self.bit_index += count << 3
-            return buffer[start_index: end_index]
+            return buffer[start_index:end_index]
         else:
-            raise EndOfBufferError('Unexpected end of buffer.')
+            raise EndOfBufferError("Unexpected end of buffer.")
 
     def read_unsigned_short(self) -> int:
-        """ Read an unsigned short (16 bit) from buffer. """
+        """Read an unsigned short (16 bit) from buffer."""
         if self.bit_index & 7:
             s1 = self.read_bits(8)
             s2 = self.read_bits(8)
@@ -190,16 +208,16 @@ class BitStream:
         return (s2 << 8) + s1
 
     def read_signed_short(self) -> int:
-        """ Read a signed short (16 bit) from buffer. """
+        """Read a signed short (16 bit) from buffer."""
         value = self.read_unsigned_short()
         if value & 0x8000:
             # 2er complement
-            return -((~value & 0xffff) + 1)
+            return -((~value & 0xFFFF) + 1)
         else:
             return value
 
     def read_unsigned_long(self) -> int:
-        """ Read an unsigned long (32 bit) from buffer. """
+        """Read an unsigned long (32 bit) from buffer."""
         if self.bit_index & 7:
             read_bits = self.read_bits
             l1 = read_bits(8)
@@ -211,11 +229,11 @@ class BitStream:
         return (l4 << 24) + (l3 << 16) + (l2 << 8) + l1
 
     def read_signed_long(self) -> int:
-        """ Read a signed long (32 bit) from buffer. """
+        """Read a signed long (32 bit) from buffer."""
         value = self.read_unsigned_long()
         if value & 0x80000000:
             # 2er complement
-            return -((~value & 0xffffffff) + 1)
+            return -((~value & 0xFFFFFFFF) + 1)
         else:
             return value
 
@@ -225,7 +243,7 @@ class BitStream:
             data = bytes(read_bits(8) for _ in range(8))
         else:  # aligned data
             data = bytes(self.read_aligned_bytes(8))
-        return struct.unpack('<d', data)[0]
+        return struct.unpack("<d", data)[0]
 
     def read_3_bits(self) -> int:
         bit = self.read_bit()
@@ -241,7 +259,15 @@ class BitStream:
         else:
             return 0  # 0
 
-    def read_bit_short(self, count=1) -> Union[int, Sequence[int]]:
+    @overload
+    def read_bit_short(self) -> int:
+        ...
+
+    @overload
+    def read_bit_short(self, count: int) -> Sequence[int]:
+        ...
+
+    def read_bit_short(self, count: int = 1) -> Union[int, Sequence[int]]:
         def _read():
             bits = self.read_bits(2)
             if bits == 0:
@@ -258,6 +284,14 @@ class BitStream:
         else:
             return tuple(_read() for _ in range(count))
 
+    @overload
+    def read_bit_long(self) -> int:
+        ...
+
+    @overload
+    def read_bit_long(self, count: int) -> Sequence[int]:
+        ...
+
     def read_bit_long(self, count: int = 1) -> Union[int, Sequence[int]]:
         def _read():
             bits = self.read_bits(2)
@@ -286,17 +320,33 @@ class BitStream:
         shifting = 0
         length = self.read_bits(3)  # or read_3_bits() ?
         while length > 0:
-            value += (self.read_unsigned_byte() << shifting)
+            value += self.read_unsigned_byte() << shifting
             length -= 1
             shifting += 8
         return value
 
+    @overload
+    def read_raw_double(self) -> float:
+        ...
+
+    @overload
+    def read_raw_double(self, count: int) -> Sequence[float]:
+        ...
+
     def read_raw_double(self, count: int = 1) -> Union[float, Sequence[float]]:
         if count == 1:
             return self.read_float()
         else:
             return tuple(self.read_float() for _ in range(count))
 
+    @overload
+    def read_bit_double(self) -> float:
+        ...
+
+    @overload
+    def read_bit_double(self, count: int) -> Sequence[float]:
+        ...
+
     def read_bit_double(self, count: int = 1) -> Union[float, Sequence[float]]:
         def _read():
             bits = self.read_bits(2)
@@ -314,16 +364,35 @@ class BitStream:
         else:
             return tuple(_read() for _ in range(count))
 
-    def read_bit_double_default(self, count: int = 1, default=0.0) -> Union[float, Sequence[float]]:
-        data = struct.pack('<d', default)
+    @overload
+    def read_bit_double_default(self) -> float:
+        ...
+
+    @overload
+    def read_bit_double_default(self, count: int) -> Sequence[float]:
+        ...
+
+    @overload
+    def read_bit_double_default(
+        self, count: int, default: float
+    ) -> Sequence[float]:
+        ...
+
+    def read_bit_double_default(
+        self, count: int = 1, default: float = 0.0
+    ) -> Union[float, Sequence[float]]:
+        data = struct.pack("<d", default)
 
         def _read():
             bits = self.read_bits(2)
             if bits == 0:
                 return default
             elif bits == 1:
-                _data = bytes(self.read_unsigned_byte() for _ in range(4)) + data[4:]
-                return struct.unpack('<d', _data)
+                _data = (
+                    bytes(self.read_unsigned_byte() for _ in range(4))
+                    + data[4:]
+                )
+                return struct.unpack("<d", _data)
             elif bits == 2:
                 _data = bytearray(data)
                 _data[4] = self.read_unsigned_byte()
@@ -332,7 +401,7 @@ class BitStream:
                 _data[1] = self.read_unsigned_byte()
                 _data[2] = self.read_unsigned_byte()
                 _data[3] = self.read_unsigned_byte()
-                return struct.unpack('<d', _data)
+                return struct.unpack("<d", _data)
             else:
                 return self.read_float()
 
@@ -342,11 +411,10 @@ class BitStream:
             return tuple(_read() for _ in range(count))
 
     def read_signed_modular_chars(self) -> int:
-        """
-        Modular characters are a method of storing compressed integer values.
-        They consist of a stream of bytes, terminating when the high bit (8)
-        of the byte is 0 else another byte follows. Negative numbers are
-        indicated by bit 7 set in the last byte.
+        """Modular characters are a method of storing compressed integer
+        values. They consist of a stream of bytes, terminating when the high
+        bit (8) of the byte is 0 else another byte follows. Negative numbers
+        are indicated by bit 7 set in the last byte.
 
         """
         shifting = 0
@@ -355,52 +423,50 @@ class BitStream:
             char = self.read_unsigned_byte()
             if char & 0x80:
                 # bit 8 set = another char follows
-                value |= ((char & 0x7f) << shifting)
+                value |= (char & 0x7F) << shifting
                 shifting += 7
             else:
                 # bit 8 clear = end of modular char
                 # bit 7 set = negative number
-                value |= ((char & 0x3f) << shifting)
+                value |= (char & 0x3F) << shifting
                 return -value if char & 0x40 else value
 
     def read_unsigned_modular_chars(self) -> int:
-        """
-        Modular characters are a method of storing compressed integer values.
-        They consist of a stream of bytes, terminating when the high bit (8)
-        of the byte is 0 else another byte follows.
+        """Modular characters are a method of storing compressed integer
+        values. They consist of a stream of bytes, terminating when the high
+        bit (8) of the byte is 0 else another byte follows.
 
         """
         shifting = 0
         value = 0
         while True:
             char = self.read_unsigned_byte()
-            value |= ((char & 0x7f) << shifting)
+            value |= (char & 0x7F) << shifting
             shifting += 7
             # bit 8 set = another char follows
             if not (char & 0x80):
                 return value
 
     def read_modular_shorts(self) -> int:
-        """
-        Modular shorts are a method of storing compressed unsigned integer values.
-        Only 1 or 2 shorts in practical usage (1GB), if the high bit (16) of
-        the first short is set another short follows.
+        """Modular shorts are a method of storing compressed unsigned integer
+        values. Only 1 or 2 shorts in practical usage (1GB), if the high
+        bit (16) of the first short is set another short follows.
 
         """
         short = self.read_unsigned_short()
         if short & 0x8000:
-            return (self.read_unsigned_short() << 15) | (short & 0x7fff)
+            return (self.read_unsigned_short() << 15) | (short & 0x7FFF)
         else:
             return short
 
-    def read_bit_extrusion(self) -> Tuple[float, float, float]:
+    def read_bit_extrusion(self) -> Sequence[float]:
         if self.read_bit():
             return 0.0, 0.0, 1.0
         else:
             return self.read_bit_double(3)
 
-    def read_bit_thickness(self, dxfversion='AC1015') -> float:
-        if dxfversion >= 'AC1015':
+    def read_bit_thickness(self, dxfversion="AC1015") -> float:
+        if dxfversion >= "AC1015":
             if self.read_bit():
                 return 0.0
         return self.read_bit_double()
@@ -418,20 +484,19 @@ class BitStream:
         # see the main Object description section for details.
         length = self.read_bit_short()
         data = bytes(self.read_unsigned_byte() for _ in range(length * 2))
-        return data.decode(encoding='utf16')
+        return data.decode(encoding="utf16")
 
     def read_text_variable(self) -> str:
-        if self.dxfversion < 'AC1018':  # R2004
+        if self.dxfversion < "AC1018":  # R2004
             return self.read_text()
         else:
             return self.read_text_unicode()
 
     def read_cm_color_cms(self) -> Tuple[int, str, str]:
-        """ Returns tuple (rgb, color_name, book_name).
-        """
+        """Returns tuple (rgb, color_name, book_name)."""
         _ = self.read_bit_short()  # index always 0
-        color_name = ''
-        book_name = ''
+        color_name = ""
+        book_name = ""
         rgb = self.read_bit_long()
         rc = self.read_unsigned_byte()
         if rc & 1:
@@ -440,25 +505,26 @@ class BitStream:
             book_name = self.read_text_variable()
         return rgb, color_name, book_name
 
-    def read_cm_color_enc(self) -> Union[int, Tuple[int, int, int, int]]:
-        """ Returns color index as int or tuple (rgb, color_handle, transparency_type, transparency).
+    def read_cm_color_enc(self) -> Union[int, Sequence[Optional[int]]]:
+        """Returns color index as int or tuple (rgb, color_handle,
+        transparency_type, transparency).
         """
         flags_and_index = self.read_bit_short()
         flags = flags_and_index >> 8
-        index = flags_and_index & 0xff
+        index = flags_and_index & 0xFF
         if flags:
             rgb = None
             color_handle = None
             transparency_type = None
             transparency = None
             if flags & 0x80:
-                rgb = self.read_bit_short() & 0x00ffffff
+                rgb = self.read_bit_short() & 0x00FFFFFF
             if flags & 0x40:
-                _, color_handle = self.read_handle()
+                color_handle = self.read_handle()
             if flags & 0x20:
                 data = self.read_bit_long()
                 transparency_type = data >> 24
-                transparency = data & 0xff
+                transparency = data & 0xFF
             return rgb, color_handle, transparency_type, transparency
         else:
             return index
@@ -468,12 +534,12 @@ class BitStream:
         if bits == 0:
             return self.read_unsigned_byte()
         elif bits == 1:
-            return self.read_unsigned_byte() + 0x1f0
+            return self.read_unsigned_byte() + 0x1F0
         else:
             return self.read_unsigned_short()
 
     def read_handle(self, reference: int = 0) -> int:
-        """ Returns handle as integer value. """
+        """Returns handle as integer value."""
         code = self.read_bits(4)
         length = self.read_bits(4)
         if code == 6:
@@ -481,10 +547,10 @@ class BitStream:
         if code == 8:
             return reference - 1
 
-        data = bytearray(b'\x00\x00\x00\x00\x00\x00\x00\x00')
+        data = bytearray(b"\x00\x00\x00\x00\x00\x00\x00\x00")
         for index in range(length):
             data[index] = self.read_unsigned_byte()
-        offset = struct.unpack('<Q', data)[0]
+        offset = struct.unpack("<Q", data)[0]
 
         if code < 6:
             return offset
@@ -493,44 +559,47 @@ class BitStream:
                 return reference + offset
             if code == 12:
                 return reference - offset
+        return 0
 
     def read_hex_handle(self, reference: int = 0) -> str:
-        """ Returns handle as hex string. """
-        return '%X' % self.read_handle(reference)
+        """Returns handle as hex string."""
+        return "%X" % self.read_handle(reference)
 
     def read_code(self, code: str):
-        """ Read data from bit stream by data codes defined in the ODA reference.
+        """Read data from bit stream by data codes defined in the
+        ODA reference.
+
         """
-        if code == 'B':
+        if code == "B":
             return self.read_bit()
-        elif code == 'RC':
+        elif code == "RC":
             return self.read_unsigned_byte()
-        elif code == 'RS':
+        elif code == "RS":
             return self.read_signed_short()
-        elif code == 'BS':
+        elif code == "BS":
             return self.read_bit_short()
-        elif code == 'RL':
+        elif code == "RL":
             return self.read_signed_long()
-        elif code == 'BL':
+        elif code == "BL":
             return self.read_bit_long()
-        elif code == 'RD':
+        elif code == "RD":
             return self.read_raw_double()
-        elif code == '2RD':
+        elif code == "2RD":
             return self.read_raw_double(2)
-        elif code == 'BD':
+        elif code == "BD":
             return self.read_bit_double()
-        elif code == '2BD':
+        elif code == "2BD":
             return self.read_bit_double(2)
-        elif code == '3BD':
+        elif code == "3BD":
             return self.read_bit_double(3)
-        elif code == 'T':
+        elif code == "T":
             return self.read_text()
-        elif code == 'TV':
+        elif code == "TV":
             return self.read_text_variable()
-        elif code == 'H':
+        elif code == "H":
             return self.read_hex_handle()
-        elif code == 'BLL':
+        elif code == "BLL":
             return self.read_bit_long_long()
-        elif code == 'CMC':
+        elif code == "CMC":
             return self.read_cm_color()
-        raise ValueError(f'Unknown code: {code}')
+        raise ValueError(f"Unknown code: {code}")
